###############################################################################
# This script collects data based on an ini file passed in as a command line
# argument.  
# This script has been refactored from spcollect_backend.pl
# The top level working directory is also passed in as an argument. 
# This script creates the following directory structure and uses that as its
# working directory.
# c:\dumps\<top_level_dir>\<inifile_name>\

use strict;
use Getopt::Long;
use POSIX qw(strftime);
use Win32::Job;
use Win32::Service;
use Win32::TieRegistry( Delimiter=>"/", ArrayValues=>0 );
use Win32;
use File::Path;
use Win32::Process; 

###########################################################################################
##########                     Set up some global values and constants           ##########
###########################################################################################

#Constants
use constant MAX_PREVIOUSSTATUS => 2;
use constant CHECK_NAVI_READY_SLEEP_VALUE => 60; # 60 seconds

#Predefined global values
my (
    $RETURN_GOOD,		#Global function return condition for success
    $RETURN_BAD,		#Global function return condition for failure
    $TRUE,			#Global true condition
    $FALSE,			#Global false condition
    $dumpdir,			#Directory where the collect zip will be saved
    $scriptname,		#The name presented to the system
    $revision,			#Current version presented to the system
    $tempdir,			#Directory where some known log files are held
    $minidumpdir,       #Directory where mini dumps are held
    $tempFileCount,    #Temporary File Counter
    $spState,

    # for use in putting headers into the output files generated by spcollect.
    $lineofstars,
    $lineofdashes,

    # Default program execution timeout
    $defaultTimeout,
    
    #dummy runlog file to make CAP work
    $dummyRunLog,

    #hold the log message that generates before the log file's name
    $message_queue
   );

$RETURN_GOOD = 0;
$RETURN_BAD = -1;
$TRUE = 1;
$FALSE = 0;
$dumpdir = "C:\\dumps";
$scriptname = "spcollect_data_gather.pl";
$revision = $ENV{'BASE_REVISION'};
$tempdir = "c:\\TEMP";
$minidumpdir = "C:\\WINNT\\Minidump";
$tempFileCount = 0;
$lineofstars = "********************************************************************************";
$lineofdashes = "--------------------------------------------------------------------------------";
$defaultTimeout = 1500000;  #  Timeout value in seconds
$message_queue = "";
$spState = "";

#Global Variables
my (
    %programHash,		#The programs and their dependencies used in the script
    %argHash,			#The arguments passed to the script
    %operatingHash,		#Commonly used paths and globals in the script
    $checkNaviReady,    #If TRUE check for navi readiness
   );

#called to create a log from the output of an executed program
sub CreateFile($$$$@);

sub GatherPromInfo();

#called to create information for each iSCSI port on the array
sub CreateiSCSIPortInfo($$$);

#called to add a header label to a file being created
sub LabelLog($$@);

#called to send output stating the current status of the collect process
sub GiveStatus($@);

#called to verify whether a specific function call should run during restricted execution
sub CheckGroup($);

#check whether a layered app is needed and if so whether the required app is enabled
sub CheckLayeredApp($);

#check file size is > 0
sub CheckValidBinary($);

#executes a system command
sub InvokeCommand($$$$$);
sub InvokeCommand_($$$$$);

sub CaptureLccLogs($$);
sub CaptureLccStats($$);
sub CaptureNorthstarLogs($$);
sub AddCreatedFileToZip($$);
sub ArrayConfigCapture($$$);
sub AdminToolCapture($$$$);
sub GetNaviLog($$$);
sub CopyFileInto($$);
sub CheckDllFile(@);
sub GenerateFslOutput($$);
sub GetTLDTrace($$);


#empty subroutine to be used as stub in the program hash
sub EmptyStub();

#subroutine to be invoked before naviseccli is executed.
sub CheckNaviReady();

#subroutine to get array serial number
sub GetArraySerialNum();

#subroutine to get SP signature
sub GetSpSignature();

#Make an entry in event log that SPC was started.
sub LogAction ;

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GetSpState
#------------------------------------------------------------------------------------------
# Determine the current state of this SP.
#
# Returns the state or UNKNOWN if the state cannot be determined.
# We currently expect a state of either "normal" or "degraded".
#------------------------------------------------------------------------------------------
# Command output looks like:
#
#    Peer SP is physically INSERTED
#    
#    This SP is in NORMAL MODE (not degraded). OR This SP is in DEGRADED MODE
#    Current reboot count: 0
#    
#    This SP is set to start HFON.
#    
#    This SP is set to start MCR.
#    
#    38 normal reboot(s) since the last panic
#    
#   Prior SP panics: (newest to oldest; 5 max)
#
# We capture the state from the line: "This SP is in NORMAL MODE..."
#------------------------------------------------------------------------------------------
sub GetSpState
  { 
    my $sp_state = "DEGRADED"; # If we cannot determine the state, default to DEGRADED
    
    # Get the current state of this SP
    my $cmd_output = `$programHash{"spstate"}{"name"}`;
    my $exit_code = $? >> 8;
    
    if ( $exit_code == 0 ) {
    
        if( $cmd_output =~ /This SP is in (\w+) MODE/ ){
            $sp_state = $1;
        }
        else{
            RunLogIt( "ERROR - Could not determine SP state. Defaulting to $sp_state. Could not parse result: $cmd_output" );
        }
    }
    else{
        RunLogIt( "ERROR - Could not determine SP state. Defaulting to $sp_state. Command failed:($exit_code) $!" );
    }
    
    return $sp_state;
  }

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : LogAction
#------------------------------------------------------------------------------------------
# Make a log entry that we are kicking off an spcollect.
#------------------------------------------------------------------------------------------
# wmb - 3/03 - Put all of this code together in its own sub
#------------------------------------------------------------------------------------------
sub LogAction
  {
    # set up the logging executable
    my $dir = "\\EMC\\Base\\$ENV{ 'BASE_REVISION' }";

    if ( -f "$dir\\errgenlog.exe" ) {
      # Make a log entry... do it here so that the logs we gather later will have this in them.
      `$dir\\errgenlog.exe log_msg info "spcollect: Rev: $revision, file: $operatingHash{zip_identifier}"`;
      `$dir\\errgenlog.exe trace_msg info "spcollect: Rev: $revision, file: $operatingHash{zip_identifier}"`;
    }

  }#end LogAction


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
#DIMS 211315
# sub : GetArraySerialNum
# description : This function uses speclcli to get the array serial number.
#------------------------------------------------------------------------------------------
sub GetArraySerialNum()
{
    my($serial_num, $line, @speclcli_resume_cmd_output, $commandName);
    $serial_num = "NOSERIALNUM";
    

    if( $ENV{'VM_SIM'} eq "1" )
    {
        my $tmp = $Registry->{"LMachine/System/CurrentControlSet/services/Flaredrv/Parameters//FlareNoResPromSerialNumber"};
        if( $tmp ne "" )
        {
            $serial_num = $tmp;
            return $serial_num;
        }
    }
    else
    {
	
        my $midplane_id = 0;

        #(1),get the id of midplane with the command "speclcli -getresume";
	#(2),get the SN with the midplane id captured in (1);
        $commandName = "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\speclcli.exe";


        my @getresume_output = `$commandName -getResume`;
	foreach my $getresume_line ( @getresume_output )
	{
	    #try to match for the line "xx - Midplane", such as "13 - Midplane"
	    if( $getresume_line =~ /(\d+)(\s*)-(\s*)midplane/i )
		{
		     $midplane_id = $1;
		     last;
		}
	}

	#write the error message into an array variable when -getResume fails
	if( $midplane_id == 0 )
	{
		my $message = "Can't find id for midplane\n";
		$message .= "output of -getresume:\n"; 
		$message .= join( "\n", @getresume_output );
		&RunLogIt( $message );

		return $serial_num;
	}

        @speclcli_resume_cmd_output = `$commandName -getResume $midplane_id 2>&1`;
	if( ! @speclcli_resume_cmd_output )
	{
		my $message = "Command returns nothing!\n";
		&RunLogIt( $message );
		return $serial_num;
	}

        foreach $line (@speclcli_resume_cmd_output)
        {
            #look for the line with "EMC Product Serial Number"
            if($line =~ /EMC\s+Product\s+Serial\s+Number\s+:\s+(.*)/)
            {
                my $serial_num_temp = $1;
                $serial_num_temp =~ s/\s+$//;
                if( $serial_num_temp =~ /\w{5}\d{9}/i )
		{
			$serial_num = $serial_num_temp;
		}
		else
		{
			my $message = "Found EMC Product Serial Number line, but the serial number is invalid!\n";
			$message .= "the returned serial number is: $serial_num_temp";
			&RunLogIt( $message );
		}
		return $serial_num;
            }
        }
	my $message = "Could not find EMC Product Serial Number line!\n";
	$message .= "the output of -getresume $midplane_id:\n";
	$message .= join("\n", @speclcli_resume_cmd_output);
	&RunLogIt( $message );
	return $serial_num;
    }   
}

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
#DIMS 211315
# sub : GetSpSignature
# description : This function uses getSpids to get the sp's signature.
#------------------------------------------------------------------------------------------
sub GetSpSignature()
{
  my($signature, $spids_line, @getspids_cmd_output);
  $signature = "NOSIG";
  
  if(uc($spState) eq "NORMAL")
  {
  	@getspids_cmd_output = `$programHash{getspids}{name}`;
  
 	 foreach $spids_line (@getspids_cmd_output)
  	{
    		 # "*" in the o/p indicates the current SP. Look for [00hexnumber]. This is the signature
    		 if($spids_line =~ /\*\s+[0-9|a-z]+:[0|1]\s+\[([0-9|a-z]+)/i)
     		{ 
	  		$signature = $1;
	  		$signature =~ s/^0+//;
	  		return $signature;
     		}
  	}
  }
  else
  {
	  return $signature;
  }
}


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GetTLDTrace
# description : This function will collect the TLD trace files.
#------------------------------------------------------------------------------------------
sub GetTLDTrace($$)
{
    my ($group, $zip) = @_;
    if(CheckGroup($group)){
        my $file_path = "C:\\temp\\DebugMerge";
    
        if(-e $file_path){
            opendir (my $FOLDER, $file_path) or RunLogIt("Unable to open specified folder: $!.");
            my $cur_time = time();
            my $min_time = $cur_time - 300;
            foreach my $file (readdir $FOLDER){
                my $create_time = (stat $file_path."\\".$file)[10];
                if($create_time < $cur_time && $create_time > $min_time){
                    InsertInventoryFileEntry($zip, $file_path."\\".$file);
                }
            }
        }
        else{
            RunLogIt("Specified folder does not exist: $file_path.");
        }
    }
    
    return ;
}


###########################################################################################
##########                             subroutines                               ##########
###########################################################################################
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CopyFileTo
# parameters : $originalFile - Path to original file
#              $targetFile   - Path to target file to append to
#------------------------------------------------------------------------------------------
# Appends the contents of the file specified by the first argument to
# the file specified by the second argument.  Creates the second file
# if it does not exist.
#------------------------------------------------------------------------------------------
sub CopyFileInto($$)
  {
    my($originalFile, $targetFile) = @_;

    #Create handles to files
    open (ORGFILE, "< $originalFile")
      or print STDERR "CopyFileInto Could not open original file ($originalFile): $!\n";
    open (TGTFILE, ">> $targetFile") 
      or print STDERR "CopyFileInto Could not open target file ($targetFile): $!\n";

    binmode(ORGFILE);
    binmode(TGTFILE);

    while (<ORGFILE>) {
      print TGTFILE;
    }

    #Close handles
    close TGTFILE;
    close ORGFILE;
  } #end CopyFileInto


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : InitProgramHash
# usage : InitProgramHash()
#------------------------------------------------------------------------------------------
# This function assembles and organizes the list of programs that will be used
# throughout this script. The list of programs along with attributes necessary
# to run each program are placed inside a global hash.
#
# The general structure of the global hash is as follows:
# programHash{
#	"progShortName" => {
#		"name" => "progPath\progName", #path to the program
#		"services" => [service0, service1, etc..], #services needed for program
#		"previousStatus" => <val representing status> #completion state of last run
#		}
#	}
#------------------------------------------------------------------------------------------
sub InitProgramHash
  {
    #adding Data Erasure Project exe to the program hash DIMS 220644
    $programHash{"ddbs_info"} = 
      {
        name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\ddbs_info.exe",
        invokemode => "normal",
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };
    
    #DIMS 211315 and DIMS 219826
    $programHash{"speclcli"} = 
      {
        name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\speclcli.exe",
        invokemode => "normal",
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };

    #adding cmd to the program hash
    $programHash{"cmd"} = 
      {
        name => "$ENV{'windir'}\\system32\\cmd.exe",
        invokemode => "normal",
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };
 
    #adding no-polling naviseccli to the program hash
    $programHash{"naviseccli"} =
      {
        name => "c:\\EMC\\Navisphere\\$ENV{'NAVISPHERE_REVISION'}\\msgbin\\NavisecCli.exe",
        invokemode => "UsesLayeredApp",
        previousStatus => 0,  #good status by default
        preInvokeSub => \&CheckNaviReady,
        defaultArgs => "-np",  # default arguments, only handled for this program (refer to CaptureLogs())
        spState => "NORMAL",
      };

    #adding mvacli to the program hash
    $programHash{"mvacli"} =
      {
        name => "c:\\EMC\\MVAEngine\\$ENV{'MVAEngine_REVISION'}\\msgbin\\mvacli.exe",
        invokemode => "UsesLayeredApp",
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "NORMAL",
      };

    #adding hostconfcli to the program hash
    $programHash{"hostconfcli"} =
      {
        name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\flare_driver\\bin\\hostconfcli.exe",
        invokemode => "normal",
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "NORMAL",
      };
      
    #adding PciHalCli to the program hash
    $programHash{"PciHalCli"} =
      {
        name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\PciHalCli.exe",
        invokemode => "normal",
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };
      
    #adding ktcons to the program hash
    $programHash{"ktcons"} =
      {
        name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\ktcons.exe", 
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };

    #adding 64-bit ktcons to the program hash
    $programHash{"ktcons64"} =
      {
        name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin_x64\\ktcons.exe", 
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };
      
    # NOTE: ktcons64 is deprecated.  DO NOT USE IT unless the
    #       msgbin_x64 directory becomes supported again.

    #adding psmtool to the program hash
    $programHash{"psmtool"} =
      {
        name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\psmtool.exe",
        invokemode => "normal",
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "NORMAL",
      };
      
    $programHash{"psmtoolRedirect"} =
      {
        name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\psmtool.exe",
        invokemode => "normal",
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "NORMAL",
      };
  
    #adding getspids to the program hash
    $programHash{"getspids"} =
      {
        name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\getSpIds.exe", 
        invokemode => "normal",
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "NORMAL",
      };
  
    #adding route to the program hash
    $programHash{"route"} =
      {
        name => "$ENV{'SystemRoot'}\\system32\\route.exe",  
        invokemode => "normal",
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };
  
    #adding ipconfig to the program hash
    $programHash{"ipconfig"} =
      {
        name => "$ENV{'SystemRoot'}\\system32\\ipconfig.exe",  
        invokemode => "normal",
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };
  
    #DIMS 226510 adding netstat to the program hash
    $programHash{"netstat"} =
      {
       name => "$ENV{'SystemRoot'}\\system32\\netstat.exe",  
       invokemode => "normal",
       previousStatus => 0,  #good status by default
       preInvokeSub => \&EmptyStub,
         spState => "ALL",
      };
  
    #DIMS 232754 adding tasklist to the program hash
    $programHash{"tasklist"} =
      {
        name => "$ENV{'SystemRoot'}\\system32\\tasklist.exe",  
        invokemode => "normal",
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };
  
    #DIMS 234059  adding fbe_cli to yhr tasklist the program hash
    $programHash{"fbe_cli"} =
      {
        name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\flare_driver\\bin\\fbe_cli.exe",  
        invokemode => "normal",
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };
  
    #MCR
    $programHash{"fbecli"} =
      {
        name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\flare_driver\\bin\\fbecli.exe", 
        invokemode => "normal",
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };
  
    #adding netsh to the program hash
    $programHash{"netsh"} =
      {
        name => "$ENV{'SystemRoot'}\\system32\\netsh.exe", 
        invokemode => "normal",
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };
  
    #adding perl to the program hash
    $programHash{"perl"} =
      {
        name => "c:\\perl\\bin\\perl.exe",
        invokemode => "normal",
        previousStatus => 0,  #good status by default
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };
  
    #adding AdminTool to program hash
    $programHash{"admintool"} =
      {
        name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\AdminTool.exe",
        invokemode => "normal",
        previousStatus => 0,
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };
  
    $programHash{"smbtool"} =
      {
        name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\smbtool.exe",
        invokemode => "normal",
        previousStatus => 0,
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };
  
    #adding spstate.exe to program hash
    $programHash{"spstate"} =
      {
        name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\spstate.exe",
        invokemode => "normal",
        previousStatus => 0,
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };
      
    #adding mlucli.exe to program hash
    $programHash{"mlucli"} =
      {
        name => "c:\\EMC\\MLUEngine\\$ENV{'MLUENGINE_REVISION'}\\msgbin\\mlucli.exe",
        invokemode => "normal",
        previousStatus => 0,
        preInvokeSub => \&EmptyStub,
        spState => "NORMAL",
      };
      
    #adding amcli.exe to program hash
    $programHash{"amcli"} =
      {
        name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\amcli.exe",
        invokemode => "normal",
        previousStatus => 0,
        preInvokeSub => \&EmptyStub,
        spState => "NORMAL",
      };
      
    #adding ddtestcli.exe to program hash
    $programHash{"ddtestcli"} =
      {
        name => "c:\\EMC\\DeduplicationEngine\\$ENV{'DEDUPLICATIONENGINE_REVISION'}\\msgbin\\ddtestcli.exe",
        invokemode => "normal",
        previousStatus => 0,
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };
    
    #adding dmetool.exe to program hash
    $programHash{"dmetool"} =
      {
        name => "c:\\EMC\\DeduplicationEngine\\$ENV{'DEDUPLICATIONENGINE_REVISION'}\\msgbin\\dmetool.exe",
        invokemode => "normal",
        previousStatus => 0,
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };

    #ading fslcli.exe into program hash
    if( exists( $ENV{'FSLENGINE_REVISION'} ) )
    {
      $programHash{"fslcli"} = 
      {
        name => "c:\\emc\\fslengine\\$ENV{'FSLENGINE_REVISION'}\\msgbin\\fslcli.exe",
        invokemode => "normal",
        previousStatus => 0,
        preInvokeSub => \&EmptyStub,
        spState => "ALL",
      };
    }
    else
    {
      my $message = "environmental variable FSLENGINE_REVISION not exist!\n";
      RunLogIt( $message );
    }
  }#end InitProgramHash

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
#DIMS 211315
#Empty subroutine does nothing
sub EmptyStub()
{
} #end EmptyStub
#------------------------------------------------------------------------------------------

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
#DIMS 211315
# sub : CheckNaviReady
# description: This function looks for the infullpoll.txt file in C:\dumps directory. 
# If it is found it waits till it goes away. This file is created by
# Navi backend when it is processing a full poll. On the subsequent first delta poll
# this file is deleted.
sub CheckNaviReady()
{
    my $didIwait = $FALSE;
    if($checkNaviReady eq $TRUE)
    {
      while (-e "c:\\dumps\\infullpoll.txt")
      {
        if($didIwait eq $FALSE)
        {
          $didIwait = $TRUE;
          &RunLogIt("Begin wait for Navi full poll.");
        }
        sleep(CHECK_NAVI_READY_SLEEP_VALUE); # sleep for 60 seconds
      }
      if($didIwait eq $TRUE)
      {
        &RunLogIt("End wait for Navi full poll.");
      }
    }
}
#------------------------------------------------------------------------------------------

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : InsertInventoryFileEntry
# description : This function makes an entry in the intermediate inventory file specific to
# the data gather ini file that is being used. The entry is a comma separated record a follows:
# <zip file name>; <outupt file name>; <temp file name>
# arg1 = <zip file name> ex : sus
# arg2 = <output file name> ex: SP<A|B>_config_info.txt
# arg3 = <temp file name>
#------------------------------------------------------------------------------------------
sub InsertInventoryFileEntry ($$$)
{
    #Assigning passed in parameters
    my($zip, $outputFile, $tempFileName) = @_;
    my($entry);
    
    #Open a handle to the inventory file
    open INV_FILE_H, ">>" . $operatingHash{data_gather_inventory_file}
      or die "Could not open run log file" . $operatingHash{data_gather_inventory_file} . "\n";
    
    $entry = $zip. ",".$outputFile.",".$tempFileName."\n";

    #Add the message to runtime log
    print INV_FILE_H $entry;

    close INV_FILE_H;

}



#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : InitValues
# description: This funciton will gather several pieces of SP and run specific
# data that will be used throughout the script. These values will be assigned 
# to a global hash.
#------------------------------------------------------------------------------------------
sub InitValues
  {
    #Local Variables
    my($cmd_output, $status);

    #System information that will be used throughout the script
    $operatingHash{base_msgbin_dir} = "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin";
    $operatingHash{mirrorview_msgbin_dir} = "c:\\EMC\\MirrorView\\$ENV{'MIRRORVIEW_REVISION'}\\msgbin";
    
   
    $operatingHash{core_driver} = "sep";
    #ini file is the passed in argument
    $operatingHash{ini_file_path} = $operatingHash{base_msgbin_dir} . "\\" . $argHash{iniFile};
   
    
    #new perl script name
    $operatingHash{spcollect_path} = $operatingHash{base_msgbin_dir} . "\\spcollect_data_gather.pl";

    #Get SP identification
    $cmd_output = `query_sp_id`;
    $status = $? >> 8;
    if ( $status == 0 ) {
      $operatingHash{sp_id} = "SPA";
    } elsif ( $status == 1 ) {
      $operatingHash{sp_id} = "SPB";
    } else {
      $operatingHash{sp_id} = "SPX";
    }

    #Setup the Hash that holds the list of programs to be used and requirements
    &InitProgramHash;

    # Determine the state of the SP
    $spState = GetSpState();

    #Set the timestamp that will be used throughout the script
    $operatingHash{file_date} = $argHash{spcTimeStamp};

    ############################################################################
    ##### Grab serial number and signature                 #####################
    ############################################################################
	$operatingHash{sp_serial} = GetArraySerialNum();
	$operatingHash{sp_sig}    = GetSpSignature();

    ##The naming of spcollect impacts  
    ##1. PeerWatch library: check PeerWatchDefs.h file 
    ##2. CAP: Check ZipProcess.java file 
    
    ## Define Identifiers, used for naming zip's, files, and header messages throughout the script ##
    $operatingHash{zip_identifier} = $operatingHash{sp_serial} . "_" . $operatingHash{sp_id} . "_" .
      $operatingHash{file_date}  . "_" . $operatingHash{sp_sig} . "_";

    $operatingHash{file_name_identifier} = $operatingHash{sp_serial}. "_" .  $operatingHash{sp_id} . "_" .
      $operatingHash{file_date} . "_" . $operatingHash{sp_sig} . "_";

    $operatingHash{log_identifier} = "#spcollect " . $operatingHash{sp_serial} . "_" . $operatingHash{sp_id} . "_" . 
       $operatingHash{file_date} . "_" . $operatingHash{sp_sig};
       
    #Send a "note" to the log file saying that we are being invoked
    &LogAction();

  }#end InitValues


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : Usage
# description: in the event that invalid command line values are passed to spcollect this
# function will display the correct command line usage framework to STDOUT
#------------------------------------------------------------------------------------------
sub Usage
  {
    #Local Variable
    my ($switchlist);

    #display the necessary usage information to the user
    $switchlist = "[-help] [-mode brief] [-checkNaviReady] [-iniFile] [-spcTimeStamp]";
    print "Usage:\n";
    print "   spcollect.pl $switchlist\n";
    print "    -help            : Display this message\n";
    print "    -mode brief      : when specified, selects subset to be captured\n";
    print "                       brief defines smallest/fastest collect mode\n";
    print "    -checkNaviReady  :When specified navi readiness is checked before navi commands are issued\n";
    print "    -iniFile <ini file for data gather> : This is the inifile that defines the data to be capured\n";
    print "    -spcTimeStamp <time stamp> : This is normally the time when SPCollect was invoked and the name of the top level directory\n";
    exit;
  }#end Usage
  
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GetGMTime
#------------------------------------------------------------------------------------------
# Returns a hash containing time and date from gmtime
# function.
#------------------------------------------------------------------------------------------
sub GetGMTime
  {
    my %gmtimeHash;
    @gmtimeHash{"sec", "min", "hour", "day", "mon", "year", "wday", "yday"} = gmtime;

    # Adjust the year value
    $gmtimeHash{year} += 1900;

    # Adjust the month value
    $gmtimeHash{mon}++;

    return %gmtimeHash;
  }# end GetGMTime

#Not to be fixed since the time stamp in the operating hash has been updated with
#the passed in argument.
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : FormatTimeHash
# parameters : $timehash - Reference to hash created by either localtime or gmtime
#------------------------------------------------------------------------------------------
# Adjusts the contents of localtime or gmtime hash with
# leading zeros for digits below 10.
#------------------------------------------------------------------------------------------
sub FormatTimeHash($)
  {
    my ($timehash, $item);
    $timehash = $_[0];
    # adjust other single digits to have a leading zero
    foreach $item (keys %{$timehash}) {
      if (${$timehash}{$item} < 10) {
        ${$timehash}{$item} = "0" . ${$timehash}{$item};
      }
    }
  }

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : FilenameDT
#------------------------------------------------------------------------------------------
# Use the GetGMTime function to get a time and date
# stamp for the construction of the output filename.
#------------------------------------------------------------------------------------------
sub FilenameDT
  {
    #Local Variables
    my(%timehash);

    # get the time
    %timehash = GetGMTime();

   # adjust other single digits to have a leading zero
    FormatTimeHash(\%timehash);

    #Construct the final time string to return
    my $filedate =  $timehash{year}. "-" . $timehash{mon} . "-" . $timehash{day} . "_" .
      $timehash{hour} . "-" . $timehash{min} . "-" . $timehash{sec};

    return $filedate;
  }				#end FilenameDT
  
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GetTempFilename
#------------------------------------------------------------------------------------------
# Returns a unique temporary file path in the working directory
#------------------------------------------------------------------------------------------
# bcm - 01/28/06 Created
#------------------------------------------------------------------------------------------
sub GetTempFilename()
  {
    my $tempFile;
    $tempFile = $operatingHash{working_dir}."\\TEMP_";
    $tempFile = $tempFile.sprintf("%03d",$tempFileCount++);

    return $tempFile;
  }

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : InitRuntimeFiles
#------------------------------------------------------------------------------------------
# Create spcollect zip filename along with the run specific log names
#------------------------------------------------------------------------------------------
sub InitRuntimeFiles()
  {
    #Create the directory that will be used to store temporary files

    $operatingHash{working_dir} = 
            $dumpdir . "\\" . "spcollect_" . $operatingHash{file_date}."\\" . $argHash{iniFile};

    if(not -e $operatingHash{working_dir}) {
      mkpath($operatingHash{working_dir});
    }

    #Set the working directory as the current directory
    chdir($operatingHash{working_dir});

    ######################## Following lines not needed  ##################
    # Create the name of the SPCollect zip file:
    #$zipHash{"name"} = $operatingHash{zip_identifier} . "data.zip";

    #Output the name of the Collect file
    #print "\n\nThe SP collect filename will be\n";
    print $operatingHash{zip_identifier} . "data.zip"."\n";
    ##########################   End lines not needed #################### 

    ###########################################################################
    ##We also need to create a log file to record what we do, stderr, and    ##
    ##stdout.								     ##
    ###########################################################################
    ## The following files are runtime log files that will be modified       ##
    ## throughout the script. They will not go onto the final list of files  ##
    ## to be zipped until they have been completed and closed                ##

    #Add entries for the log stdout, stderr, and "log" file into the operating Hash
    #All these files live in the current working directory
    $operatingHash{log_file} =  &GenUniqueLogName("runlog.txt");
    $operatingHash{stdo_file} = &GenUniqueLogName("stdout_log.txt");
    $operatingHash{stde_file} = &GenUniqueLogName("stderr_log.txt");
    $operatingHash{data_gather_inventory_file} = &GenUniqueLogName($argHash{iniFile}."_inventory.txt");
    
    $dummyRunLog = $dumpdir."\\".$operatingHash{log_file};
    open DUMMY_LOG_FILE_H, ">>$dummyRunLog"
      or die "Could not open run log file" . $dummyRunLog . "\n";
    print DUMMY_LOG_FILE_H "-----This is a dummy log file to satisfy CAP requirement--------\n";
    close DUMMY_LOG_FILE_H;


  }	#end InitRuntimeFiles
 
 
 #~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : RestoreStd
#------------------------------------------------------------------------------------------
# This function takes the redirected STDERR stream and puts it back to its normal 
# orientation.
#------------------------------------------------------------------------------------------
# wmb - 5/19/03 - Created
# dad - 11/04 - STDOUT removed from this function
#------------------------------------------------------------------------------------------
sub RestoreStd
  {
    #Close the redirected stream
    close STDERR;
	
    #Reconnect the stream
    open (STDERR, ">&OLDSTDERR") or die "Can't restore STDERR: $!";

    #Close the old file handle
    close OLDSTDERR;
  }	#end RestoreStd


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : RedirectStd
#------------------------------------------------------------------------------------------
# This function redirects the STDERR stream.
#------------------------------------------------------------------------------------------
# wmb - 5/19/03 - Created
# dad - 11/04 - STDOUT removed from this function
#------------------------------------------------------------------------------------------
sub RedirectStd
  {
    #First, duplicate the old STDERR stream so that we can restore it later
    open (OLDSTDERR, ">&STDERR") or die "Couldn't dup STDERR: $!";

    #Now redirect the stream

    open (STDERR, ">> $operatingHash{stde_file}") or die "Can't open $operatingHash{stde_file}: $!";

    #Make the streams unbuffered
    select(STDERR); $| = 1;
  } #end RedirectStd
 
 #~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : LogSystemTime
#------------------------------------------------------------------------------------------
# Constructs a string displaying the GMT time and the local
# time reported by Perl's gmtime and localtime respectively.
#------------------------------------------------------------------------------------------
# bcm - 01/20/06 - created
#------------------------------------------------------------------------------------------
sub LogSystemTime
  {
    #local variables
    my(%localhash,$localTimeZone, $timeDif,%gmthash,$timeString,$item,$DST);

    # Get GMT Time
    %gmthash = GetGMTime();
    # Get Local Time
    @localhash{"sec", "min", "hour", "day", "mon", "year", "wday", "yday", "isdst"} = localtime;

    #Format time hashes
    FormatTimeHash(\%gmthash);
    $localhash{mon}++;
    $localhash{year} += 1900;
    if($localhash{isdst} == 1) {
      $localhash{hour}--;
      $DST = " DST"
    }
    FormatTimeHash(\%localhash);

    # Get timezone using POSIX strftime function
    $localTimeZone = strftime("%Z",localtime);
    $timeDif = $localhash{hour} - $gmthash{hour};

    # Create GMT/Local Time String
    $timeString = "System Time: $gmthash{year}-$gmthash{mon}-$gmthash{day} " . 
    "$gmthash{hour}:$gmthash{min}:$gmthash{sec} GMT" .
    "    $localhash{year}-$localhash{mon}-$localhash{day} " .
    "$localhash{hour}:$localhash{min}:$localhash{sec} (GMT $timeDif:00) $localTimeZone $DST";

    RunLogIt($timeString);
  }
  
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GatherVersionInfo
#------------------------------------------------------------------------------------------
# Figure out which version and dumpdir we are using, and put that in our records.
#------------------------------------------------------------------------------------------
# wmb - 3/03 - Put all of this code together in its own sub
#------------------------------------------------------------------------------------------
sub GatherVersionInfo
  {
    #local variable
    my($file_name);

    #create a unique name for the log
    $file_name = &GenLogName("collect_name_rev.txt");
	
    #Create an entry for the version info file in the operating Hash
    $operatingHash{fname_name_rev} = $file_name;

    #Add header and timestamp to the version info file
    &HiliteLogMessage($operatingHash{fname_name_rev}, 
		      "Name of collect script: $scriptname",
		      "Version of collect script: $revision");
    &LogMessage($operatingHash{fname_name_rev}, 
		"Start time of operation: " . $operatingHash{file_date});
  }#end GatherVersionInfo
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : PreRunSetup
#------------------------------------------------------------------------------------------
#The purpose of this function is to allow us to check a number of conditions
#at/near the beginning of run-time. We also initialize many values and files used later.
#------------------------------------------------------------------------------------------
#wmb - Created 3/03
#------------------------------------------------------------------------------------------
sub PreRunSetup
  {
    #test for valid command line arguments 
    unless (GetOptions(\%argHash, 'help', 'mode=s', 'checkNaviReady', 'spcTimeStamp=s','iniFile=s'))

    {
	  &Usage();
    }

    #provide help on usage if requested
    &Usage() if($argHash{'help'} || $argHash{'h'});
    
    #setup global values that will be used throughout the script
    &InitValues();

    #Setup some of the files and and the working directory
    &InitRuntimeFiles();

    #Create top level log containing version and timestamp info
    &GatherVersionInfo();

    #Pipe STDERR to other streams
    &RedirectStd();

    #Note GMT and Local Time in run log
    LogSystemTime();

    #Log Run mode type
    my $mode;
    if (defined $argHash{'mode'}) {
      $mode = $argHash{'mode'};
    } else {
      $mode = "Full";
    }
    
    if (defined $argHash{'checkNaviReady'}) {
      $checkNaviReady = $argHash{'checkNaviReady'};
    } else {
      $checkNaviReady = 0;
    }
    
    
    
    RunLogIt("SPCollect Mode: $mode");
    RunLogIt("SP state is: $spState");
    RunLogIt("Navi readiness check : $checkNaviReady");
    RunLogIt("Working dir is: $operatingHash{working_dir}");
    RunLogIt("Process working dir is: $operatingHash{working_dir_ini}");
    RunLogIt("Commandline arguments: @{[ %argHash ]}");
  }  

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub - CreateiSCSIPortInfo
#------------------------------------------------------------------------------------------
# Reads input from the passed in log file matching for port ID information to create a list
# of ports to query with netsh.  Matches:
#   SP: (A|B)
#   Port ID: (PortID)
#
# This is a compromise from the way SPCollect traditionally operates (i.e. blindly issuing
# commands) to help reduce SPCollect execution time on system with no iSCSI ports and to
# ensure all ports are queried when they exist.
#
#------------------------------------------------------------------------------------------
# bcm - 11/14/07 - Created (DIMS: 121414, 183408)
#------------------------------------------------------------------------------------------
sub CreateiSCSIPortInfo($$$)
{
  my ($mode, $zip, $file) = @_;
  my $fileName = &GenLogName($file);

  open ISCSI, "< $fileName" or die "Could not open $fileName\n";
  my @lines = <ISCSI>;
  close ISCSI;

  # Get iSCSI interface names from netsh interface
  foreach  my $line (@lines) {
    chomp($line);
    #Configuration for interface "iSCSIPort_59_4_2"
    if ($line =~ m/^Configuration\s+for\s+interface\s+\"(iSCSIPort_.*)\"$/i) {
      &CreateFile($mode, $zip, $file, "netsh", "routing ip show filter $1");
    }
  }

  return $TRUE;
} # end GetiSCSIFiles


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub - GatherPromInfo
#------------------------------------------------------------------------------------------
# Use ktcons to collect prom info
#------------------------------------------------------------------------------------------
# Uses ktcons !getprominfo commands.  Ultimately, a file called prom_info.txt will be
# placed in the list of files for sus.zip file.  Produces one ktcons input file per bus
# to get around a ktcons limitation on the number of commands that can be put into an
# input file.  Each enclosure on the bus produces a separate ktcons log file so we can
# put a header on it.  Otherwise, it is hard to visually separate the components.
#------------------------------------------------------------------------------------------
sub GatherPromInfo()
{
    my @ktcons_log_files;
    my $first_ktcons_ini_path = ".\\_ktcons.ini";
    my $pe_info_path = ".\\prom_info_pe.txt";

    unlink $first_ktcons_ini_path if (-e $first_ktcons_ini_path);

    # Processor enclosure prom info
    open KTCONS_LOG_FILE, ">$pe_info_path" or die "Error: Can't open $pe_info_path file: $1\n";
    print KTCONS_LOG_FILE "Information for PEs\n\n";
    close KTCONS_LOG_FILE;

    push(@ktcons_log_files, $pe_info_path);

    open KTFILE, ">$first_ktcons_ini_path" or die "Error: Can't open $first_ktcons_ini_path file: $!\n";
    print KTFILE "!load ppdbg.dll\n";
    print KTFILE ".logappend $pe_info_path\n";
    foreach my $prom_id (1 .. 24) {
      print KTFILE "!getprominfo -1 $prom_id\n";
    }
    print KTFILE ".logclose $pe_info_path\n";

    # ktcons has a limit to the size of the _ktcons.ini file, so we have to break them up
    # into smaller files.  We will do one bus for each _ktcons.ini file.
    foreach my $bus (0 .. 7) {
      my $starting_enclosure = $bus * 8;
      my $ktini = "_ktcons_$bus" . ".ini";
      my $ktini_path = ".\\$ktini";

      # Append previous ini with source of this new ini
      print KTFILE ".source $ktini_path\n";
      close KTFILE;

      open KTFILE, ">$ktini_path" or die "Error: Can't open $ktini_path File: $!\n";

      # Get prom info for each enclosure on the bus
      foreach my $bus_enclosure (0 .. 7) {
        my $enclosure_id = $starting_enclosure + $bus_enclosure;
        my $prom_info_path = ".\\prom_info_".$enclosure_id.".txt";

        open KTCONS_LOG_FILE, ">$prom_info_path" or die "Error: Can't open $prom_info_path File: $!\n";

        print KTCONS_LOG_FILE "\nInformation for bus $bus enclosure $bus_enclosure\n\n";
        close KTCONS_LOG_FILE;

        push(@ktcons_log_files, $prom_info_path);

        print KTFILE ".logappend $prom_info_path\n";

        # Prom FRU info
        foreach my $prom_id (1 .. 5) {
          print KTFILE "!getprominfo $enclosure_id $prom_id\n";
        }

        print KTFILE ".logclose\n";
      }
    }

    print KTFILE "q\n";
    close KTFILE;
    # Invoke ktcons to do the work.  Can't use &CaptureKtconsLog because I need
    # multiple ktcons input (.ini) files, and for ease of viewing the captured
    # spcollect data, each bus logs to a different ktcons log file (which must
    # be concatenated into one file to then be zipped).  It is easier to have one
    # file to view, but since ktcons cannot write an arbitrary string to the log
    # file (at least, that I know of), we need a header on each and every single
    # log file.
    &InvokeCommand($programHash{"ktcons"}{name}, "-il -f $first_ktcons_ini_path", "",
                   $programHash{"ktcons"}{preInvokeSub},"sus");

    # Now, concatenate all the ktcons log files into one bug file.  Can't
    # use &CaptureLogs because I don't want the header or a command to be
    # executed.
    my $prom_path = &GenLogName("prom_info.txt");

    &HiliteLogMessage($prom_path, "Information from KTCONS", "PROM info");

    open  PROM_FILE, ">$prom_path" or die "Error: Can't open $prom_path File: $!\n";
    print PROM_FILE "$lineofstars\n";
    print PROM_FILE "PROM Information\n";
    print PROM_FILE "$lineofdashes\n";
    close PROM_FILE;


    foreach my $file (@ktcons_log_files) {
      #Insert an entry into the intermediate inventory file
      &InsertInventoryFileEntry ("sus",$prom_path,$file);
      CopyFileInto($file,$prom_path);
      unlink "$file";
    }

    foreach my $bus (0 .. 7) {
      my $ktini_path = ".\\_ktcons_".$bus.".ini";
      unlink "$ktini_path";
    }
  return $TRUE;
}

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub :  CaptureFbecliOutput
#------------------------------------------------------------------------------------------
# Gather fbecli output
#------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------
sub CaptureFbecliOutput
  {
    #Local Variables
    my ($program, $zipIdentifier, $fileName, @commandList) = @_;
    my ($command, $returnVal, $tempFile);
    
    $tempFile = GetTempFilename();

    #Setup all of the necessary parameters to create the log file
    open SCRIPT, ">_fbecli_cmd.txt" or print STDERR "Could not open _ktcons.ini for writing: $!\n";
    foreach $command (@commandList) {
      print SCRIPT "$command\n";
    }
    print SCRIPT "quit\n";
    close SCRIPT;

    #Open a handle to the runtime log
    open LOG_FILE_H, ">>$operatingHash{log_file}"
      or die "Could not open run log file" . $operatingHash{log_file} . "\n";

    #Print a copy of _ktcons.ini in STDERR
    print STDERR "----======== _fbecli_cmd.txt -> $fileName ========----\n";
    print LOG_FILE_H "----======== _fbecli_cmd.txt -> $fileName ========----\n";
    open SCRIPT, "<_fbecli_cmd.txt" or print STDERR "Could not open _fbecli_cmd.txt for reading: $!\n";
    while(<SCRIPT>) {
      print STDERR;
      print LOG_FILE_H;
    }
    close SCRIPT;
    print STDERR "----==========================================----\n";
    print LOG_FILE_H "----==========================================----\n";
    close(LOG_FILE_H);

    #Create the log file that was just setup
    $returnVal = &InvokeCommand($programHash{$program}{name}, "k a o\"$tempFile\" z\"_fbecli_cmd.txt\"", "",
                                $programHash{$program}{preInvokeSub});
    unlink "_fbecli_cmd.txt";

    #Copy results from temp file
    CopyFileInto($tempFile, $fileName);
    
    &InsertInventoryFileEntry ($zipIdentifier,$fileName,$tempFile);      
    return $returnVal;
  }

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub :  CaptureKtconsLog
#------------------------------------------------------------------------------------------
# Gather all of the Ktrace data together
#------------------------------------------------------------------------------------------
# bcm - 02/03/06 ktcons now writes to temp file (DIMS138667), copy _ktcons.ini to STDERR
# vtk - 11/21/07 add a parameter so we can specify either "ktcons" or
#                "ktcons64" to support both 32 and 64 bit operation as
#                needed.  Converted the original routine to a helper routine
#                and added wrapper calls for 32-bit and 64-bit ktcons.
#------------------------------------------------------------------------------------------

sub CaptureKtconsLog
  {
    return CaptureKtconsLog_helper( "ktcons", @_ );   # 32-bit
  }

sub CaptureKtcons64Log
  {
    return CaptureKtconsLog_helper( "ktcons64", @_ ); # 64-bit
  }

sub CaptureKtconsLog_helper
  {
    #Local Variables
    my ($program, $zipIdentifier, $fileName, @commandList) = @_;
    my ($command, $returnVal, $tempFile);
    
    $tempFile = GetTempFilename();

    #Setup all of the necessary parameters to create the log file
    open SCRIPT, ">_ktcons.ini" or print STDERR "Could not open _ktcons.ini for writing: $!\n";
    print SCRIPT "!load ppdbg\n";
    print SCRIPT ".logappend $tempFile\n";
    foreach $command (@commandList) {
      print SCRIPT "$command\n";
    }
    print SCRIPT ".logclose\n";
    print SCRIPT "q\n";
    close SCRIPT;

    #Open a handle to the runtime log
    open LOG_FILE_H, ">>$operatingHash{log_file}"
      or die "Could not open run log file" . $operatingHash{log_file} . "\n";

    #Print a copy of _ktcons.ini in STDERR
    print STDERR "----======== _ktcons.ini -> $fileName ========----\n";
    print LOG_FILE_H "----======== _ktcons.ini -> $fileName ========----\n";
    open SCRIPT, "<_ktcons.ini" or print STDERR "Could not open _ktcons.ini for reading: $!\n";
    while(<SCRIPT>) {
      print STDERR;
      print LOG_FILE_H;
    }
    close SCRIPT;
    print STDERR "----==========================================----\n";
    print LOG_FILE_H "----==========================================----\n";
    close(LOG_FILE_H);

    #Create the log file that was just setup
    $returnVal = &InvokeCommand($programHash{$program}{name}, " -il -f _ktcons.ini", "",
                                $programHash{$program}{preInvokeSub});
    unlink "_ktcons.ini";

    #Copy results from temp file
    CopyFileInto($tempFile, $fileName);
    
    &InsertInventoryFileEntry ($zipIdentifier,$fileName,$tempFile);      
    return $returnVal;
  }	#end CaptureKtconsLog

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
#------------------------------------------------------------------------------------------
#The purpose of this function is to check whether the dll file to be loaded exist in the 
#specified directory.
#return values: $FALSE if the specified dll file does not exist, or $TRUE if the specified 
#dll file exists.
#------------------------------------------------------------------------------------------
sub CheckDllFile(@)
{
	my @cl_args = @_;
	foreach my $cl_arg ( @cl_args )
	{
		#the context try to match is "!load c:\\...\\xxx.dll"
		if( $cl_arg =~ /^(!load)\s(c:.+\w+\.dll)/i )
		{
			my $dll_path = $2;
			if( not -e $dll_path )
			{
				&RunLogIt( "$cl_arg does not exist!" );
				return $FALSE;
			}
		}
		#last;
	}
	return $TRUE;
}   #end of CheckDllFile

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CreateFile
# usage : CreateFile($group, $zip, $file_name, $program, @cl_args)
# parameters : $group - The restricted execution groups that this function call belongs to
#              $zip - The nice name of the zip file where the file will be copied to.
#              $file_name - The name of the file to hold the output of the executed program
#              $program - the program to be executed
#              @cl_args - any command line arguments to be passed to the executed program
# return values: This function will return $FALSE if the given file could not be created for
# any reason, or returns $TRUE on a success.
#------------------------------------------------------------------------------------------
# This function will execute the passed in program with the given command line options and 
# then save the output of the program to the passed in file name. This file will then be 
# added to the list of created files for the passed in zip file.
#------------------------------------------------------------------------------------------
sub CreateFile($$$$@)
  {

    #locals
    my ($cl_temp, $cl_string, $counter, $proceed, $message, $label_for_log, $funcReturn);
    my ($invokemode);

    #value to be returned
    my $to_return = $TRUE;

    #Setting locals to the passed in parameters
    my($group, $zip, $file_name, $program, @cl_args) = @_;

    if (CheckGroup($group)) {
      #Check to see that the proper number of parameters were passed to the function
      if (@_ < 4) {
	    $to_return = $FALSE;
      } else {

	  #Replacing any environment variable names with values in the command line args
	  for ($counter = 0; $counter < @cl_args; $counter++) {
	    $cl_args[$counter] =~ s/%([^%]*)%/$ENV{$1} || "%$1%"/eg;
	}
	#if the dll file does not exist, exit this function
	if( CheckDllFile( @cl_args ) == $FALSE )
	{
		return $FALSE;
	}

	#See if this program has failed to many times and if so skip its execution
	if ($programHash{$program}{previousStatus} < MAX_PREVIOUSSTATUS) {

          $invokemode = $programHash{$program}{invokemode};
	
	  #check to see if the program should work
	  if (&CanExecute($program)) {
	    #Set value to return to default
	    $funcReturn = $RETURN_BAD;

	    if ($program ne "zip") {
	      #Creating the complete name for the log file
	      $file_name = &GenLogName($file_name);

		  #Add a header to the file being created
		  if(!(-e $file_name)){
		 	#adding the log identification string  
		 	&LogMessage($file_name, $operatingHash{log_identifier}, $lineofstars);     
		    &HiliteLogMessage($file_name, "Information from $program");
            &InsertInventoryFileEntry ($zip,$file_name,"");      
		   }
	    }

	    ####
	    #### Each program that can be executed by spcollect has a different set of rules
	    #### to follow. As a result a different path is followed depending on which
	    #### program is called for
	    ####

	    #Ktcons calls have their own subroutine
	    if ($program eq "ktcons") {
	      $funcReturn = &CaptureKtconsLog($zip, $file_name, @cl_args);
	    } elsif ( $program eq "ktcons64" ) {
	      $funcReturn = &CaptureKtcons64Log($zip, $file_name, @cl_args);
	    } elsif ( $program eq "fbecli" ) {
	      $funcReturn = &CaptureFbecliOutput($program, $zip, $file_name, @cl_args);
	    }

	    #Function calls that MAY rely on layered app information need to be checked
	    elsif ($invokemode eq "UsesLayeredApp") {
	      $funcReturn = &UsesLayeredApp($zip, $file_name, $program, @cl_args);
	    } elsif ($invokemode eq "normal") {

	      foreach $cl_temp (@cl_args) {
                &HiliteLogMessage($file_name, "$program $cl_temp");

                #psmtool has a special invocation case
                if($program eq "psmtool") {
                  $funcReturn = &InvokeCommand($programHash{$program}{name},
                                                        "$cl_temp $file_name", "",
                                                        $programHash{$program}{preInvokeSub},$zip);
                  &InsertInventoryFileEntry($zip, $file_name, "");                                     
                } elsif ($program eq "psmtoolRedirect") {
                  $funcReturn = &InvokeCommand($programHash{$program}{name}, "$cl_temp", $file_name,
                                                $programHash{$program}{preInvokeSub},$zip);
                } else {
                  $funcReturn = &InvokeCommand($programHash{$program}{name},
                  "$cl_temp", $file_name, $programHash{$program}{preInvokeSub},$zip);
                }
	      }
	    } 
	    #The program variable is not one that this script currently knows about
	    else {
	      #adding a note to the runtime log that a bad call was made out of the ini file
	      &GiveStatus("", "Error: A call to the program $program was made out of the ini file.",
			  "The program in question is not known to the spcollect.");
	    }

	    #Setting up return values
	    if ($funcReturn == $RETURN_GOOD) {
	      $to_return = $TRUE;
	    }
	  }

	  #One of the necessary services for the program is not running
	  else {
	    #signal that the program could not run this time
	    $programHash{$program}{previousStatus}++;
	  }
	}

	#The previous_status count is too high which means that the program failed
	#to run multiple times and will now be skipped for the rest of the execution of this program.
	else {
	  #Note that the program will be skipped for the rest of spcollect
	  &GiveStatus("", "The program $program failed to run multiple times.",
		      "Gathering of data from this program is being skipped.");
	    }
      }
    }
	#Setting up return values
	if ($funcReturn == $RETURN_GOOD) {
	  $to_return = $TRUE;
	}
	return $to_return;
  }	#end CreateFile

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : UsesLayeredApp
# usage : UsesLayeredApp($zip, $file_name, $program, @cl_args)
# parameters : $zip - The nice name of the zip file where the file will be copied to.
#              $file_name - The name of the file to hold the output of the executed program
#              $program - the program to be executed
#              @cl_args - any command line arguments to be passed to the executed program
#------------------------------------------------------------------------------------------
# This function checkes to see if the program being used is naviseccli,
# or mvacli (the only three programs part of spcollect that are currently known
# to use layered app information).  Checks to see if the log being generated requires the 
# use of a layered app.  A check is needed to ensure that layered apps being used are 
# enabled or erroneous errors may be generated in the SP's ktraces. Next  If all checks are
# passed then the command is executed and the output is placed in the desired file, and if 
# not then a proper log message is put in place of the command output.
#------------------------------------------------------------------------------------------
# bcm - 07/10/07 - Added NaviSecCLI support
#------------------------------------------------------------------------------------------
sub UsesLayeredApp
  {
    #assigning passed in variables
    my($zip, $file_name, $program, @cl_args) = @_;

    #locals
    my($counter, $message, $returnVal, $cl_temp, @status);

    #initialize the return value
    $returnVal = $RETURN_BAD;

    #Checks and execution for mvacli
    if ($program eq "mvacli") {
      #Check whether mva is enabled and whether the command can continue
      @status = &CheckLayeredApp($program);

      #Execute the command if mva is enabled
      if ($status[0] == $TRUE) {
        foreach $cl_temp (@cl_args) {
          &HiliteLogMessage($file_name, "$program $cl_temp");
          $returnVal = &CaptureLogs($zip, $file_name, $programHash{$program}{name}, $cl_temp);
        }
        if (-e $file_name) {
         &InsertInventoryFileEntry ($zip,$file_name,"");      
        }
      }
      #The layered app being checked is not installed and will be noted in the file
      else {
        print $status[1] . "\n";
        &HiliteLogMessage($file_name, $program);
        &LogMessage($file_name, $status[1]);
        &InsertInventoryFileEntry ($zip,$file_name,"");      
      }
    }

    #checks and execution for naviseccli calls
    elsif($program eq "naviseccli") {
     ##
     ## naviseccli can obtain information from layered apps. 
     ## Erroneous critical errors can be generated by querrying layered apps that are not
     ## installed/enabled. Therefore, we must check to see if the information being 
     ## gathered by each argument requires a layered app and if so whether the app is 
     ## installed/enabled.
     foreach $cl_temp (@cl_args) {
       $returnVal = $RETURN_BAD;

       #check whether argument requires a layered app	
       @status = CheckLayeredApp($cl_temp);

       #if no layered app is needed or if it is enabled the proceed
       if ($status[0] == $TRUE) {
         # Append default arguments
         $cl_temp = $programHash{$program}{defaultArgs}." ".$cl_temp
           if (defined($programHash{$program}{defaultArgs}));

         $returnVal = &CaptureLogs($zip, $file_name, $programHash{$program}{name}, $cl_temp);
         #
         # Now add file to list to be added to the appropriate zip file
         #
         if (-e $file_name) {
          &InsertInventoryFileEntry ($zip,$file_name,"");      
         }
       }

       #the layered app being checked in question is not installed and will be noted in the file
       else {
         $returnVal = $RETURN_GOOD; #layered app not enabled, but the command didn't fail itself
         print $status[1] . "\n";
         &HiliteLogMessage($file_name, $program . " " . $cl_args[$counter]);
         &LogMessage($file_name, $status[1]);
         &InsertInventoryFileEntry ($zip,$file_name,"");      
       }
     }
   } else {
     $message = "The program being used is not recognized by spcollect!\n"; 
     print $message;
     &HiliteLogMessage($file_name, $program);
     &LogMessage($file_name, $message);
     &InsertInventoryFileEntry ($zip,$file_name,"");      
   }

    #
    #  Unfortunately, only the last returnVal is returned and reported...
    #
    return $returnVal;
  }	#end UsesLayeredApp


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CanExecute
# usage : CanExecute($program)
# parameters : $program - the program that is going be tested
#------------------------------------------------------------------------------------------
# When an SP is in a 'normal' state, we assume that all required services are running
# and therefore, all required data collection commands can be run. However, when the 
# SP is in a 'degraded' state, some services will not be running and some data collection
# commands cannot be run. This function compares the SP state requirements of each program,
# as defined in $programHash to the current state.
#
# Returns $TRUE if the program can be run given the SP state and false otherwise.
#------------------------------------------------------------------------------------------
sub CanExecute
  {
    my $program = shift;
    my $can_execute = $TRUE;	

    #Check the passed program against the program registry	
    if ( exists $programHash{$program}) {

      # Check to see if this program has SP state restrictions
      if( uc($programHash{$program}{'spState'}) ne "ALL" ){
        
        if( uc($programHash{$program}{'spState'}) ne uc($spState) ){
            $can_execute = $FALSE;	
            GiveStatus("", "Skipping program: $program because SP is in a $spState state");
        }
      }
    }
    else {
      # This program is not part of the programHash
      GiveStatus("", "The requested program is not in the programHash: $program");
    }
    return $can_execute;
  }	#end CanExecute

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GatherAllData
#------------------------------------------------------------------------------------------
# This is the master "gather" function.  It is called from main and controls the order
# in which all of the data is gathered.  Any new data that must be collected should be
# its own sub function and be called from this function.
#------------------------------------------------------------------------------------------
# wmb - 3/03 - Created
#------------------------------------------------------------------------------------------
sub GatherAllData
  {
    #locals
    my($file_line, @commands);
    my($path, $newPath, $dirname);

    #open the external file that contains most function calls
    open(INIFILE, $operatingHash{ini_file_path}) or
      &GiveStatus("", "The ini file could not be opened");

    #Gather all of the information from the file.
    @commands = <INIFILE>;
    close INIFILE;

    #Parse the information and extract the function calls.
    $file_line = join("", @commands);
    $file_line =~ s/\cJ|\cI|\n//g; #replace carriage return/line feed combos with a newline
    #The regex below does the following:
    # Looks for valid command calls in the external file
    # & - look for an ampersand 
    # \w+ - look for some word
    # \( - look for an open parethese
    # (?:)+? - find the regex inside the parentheses 0 or more times, but do not return it
    # .*? - look for at 0 or more non-whitespace characters (minimally)
    # \n* - look for zero or more new lines
    # \); - look for closing paren and then a semicolon
    # g - repeat this until the end of the stream
    # The regex is set to an array so that each successful find of the regex will have its
    # own discrete entry in the array.
    @commands = ($file_line =~ /&\w+\((?:.*?\n*)+?\);/g);

    #execute the function calls one at a time
    foreach $file_line (@commands) {
      if ((eval $file_line) == $FALSE) {
        #note that the line from the ini file was not executed correctly
        &GiveStatus("", "\tError: The line:", "\t$file_line", 
          "\tWas not executed successfully");
      }
    }

    &InsertInventoryFileEntry ("top",$operatingHash{log_file},"");
    &InsertInventoryFileEntry ("top",$operatingHash{stdo_file},"");
    &InsertInventoryFileEntry ("top",$operatingHash{stde_file},"");


    $dirname = GenLogName("dir.txt");
    &InvokeCommand($programHash{'cmd'}{name}, "/c dir c:\\ /s /os", $dirname,
                   $programHash{'cmd'}{preInvokeSub},"sus");
    &InsertInventoryFileEntry ("sus",$dirname,"");      
}

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : AdminToolCapture
#------------------------------------------------------------------------------------------
# Captures AdminTool output.
#------------------------------------------------------------------------------------------
sub AdminToolCapture($$$$)
  {
    my ($mode, $zip, $file, $cmd) = @_;

    &InvokeCommand($programHash{"admintool"}{name}, $cmd, "",
                   $programHash{"admintool"}{preInvokeSub},$zip);

    if (-e $file) {
      my $pos = rindex($file, "\\") + 1;
      my $newFile = substr($file, 0, $pos) . $operatingHash{sp_id} . "_" . substr($file,$pos);
      rename($file, $newFile);
      &GiveStatus("", "Status: gathering $newFile");
      &InsertInventoryFileEntry ($zip,$newFile,"");      
      
    } else {
      &GiveStatus("", "Status: unable to create $file");
    }

    return $TRUE;
  }


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GenerateFslOutput
#------------------------------------------------------------------------------------------
#This code will invoke the fslcli.exe to generate the Pool Service output
#------------------------------------------------------------------------------------------
sub GenerateFslOutput($$)
{
	my ($zip, $cmd) = @_;
	if(exists($programHash{"fslcli"}{name}))
	{
	&InvokeCommand($programHash{"fslcli"}{name}, $cmd, "", $programHash{"fslcli"}{preInvokeSub}, $zip);
	return $TRUE;
	}
	else
	{
		return $FALSE;
	}
}

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : CaptureLccLogs
#------------------------------------------------------------------------------------------
# This code searches c:\dump for all files that match m/SP[ABab].*yukon_log_encl.*\.xml
# All matched files are added to the lcc zip and deleted
#------------------------------------------------------------------------------------------
sub CaptureLccLogs($$)
  {
    #my ($group, $zip) = @_;
    #my ($path);

# Deleted code invoking spcyukoncollect.exe to gather yukon enclosure data (Klondike FC drives).

    return $TRUE;
  }

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : ArrayConfigCapture
#------------------------------------------------------------------------------------------
# Usage: &ArrayConfigCapture($mode, $zip)
# Purpose: Capture array configuration using NaviSecCLI arrayconfig -capture.
# Parameters: $source      - Source file to copy.
#             $destination - Destination file to copy to (overwrites if file exists).
#------------------------------------------------------------------------------------------
sub ArrayConfigCapture($$$)
  {
    my ($mode, $zip, $file) = @_;

    $file = "$operatingHash{working_dir}\\".&GenLogName($file);
    &InvokeCommand("$programHash{naviseccli}{name}", "-np arrayconfig -capture -output $file -o", "",
                    $programHash{naviseccli}{preInvokeSub},$zip);
    &InsertInventoryFileEntry ($zip,$file,"");      
    return $TRUE;
  }

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GetNaviLog
#------------------------------------------------------------------------------------------
# Usage: &GetNaviLog($mode,$zip, $file)
# Purpose: Get navi log using convertemlog.
# Parameters: $mode      - Mode of SPC invocation.
#             $zip       - Zip file into which the log o/p will be put
#             $file      - Name of the o/p file
#------------------------------------------------------------------------------------------
sub GetNaviLog($$$)
  {
    my ($mode, $zip, $file) = @_;

    $file = "$operatingHash{working_dir}\\".&GenLogName($file);
    &InvokeCommand("$programHash{naviseccli}{name}", "-np convertemlog -write $file", "",
                    $programHash{naviseccli}{preInvokeSub},$zip);
    &InsertInventoryFileEntry ($zip,$file,"");      
    return $TRUE;
  }

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : RunLogIt
#------------------------------------------------------------------------------------------
# This function accepts a string to log to the log file.  It will add a time stamp to
# the entry also.
# Arguments:
# The first argument should be the string to log.
#------------------------------------------------------------------------------------------
# wmb - 5/19 - Created
#------------------------------------------------------------------------------------------
sub RunLogIt
  {
    my $log_string = $_[0];

    if( exists $operatingHash{log_file} )
    {
     #Open a handle to the runtime log
        open LOG_FILE_H, ">>" . $operatingHash{log_file}
           or die "Could not open run log file" . $operatingHash{log_file} . "\n";

        if( $message_queue )
        {
	     print LOG_FILE_H $message_queue;
	     $message_queue = "";
        }

        #Local Variables
        my (%timehash, $cur_time);

        #Get the current time and date into a local data structure
        %timehash = GetGMTime();

        # adjust other single digits to have a leading zero
        FormatTimeHash(\%timehash);

        #Format the time as desired
        $cur_time = "[$timehash{hour}:$timehash{min}:$timehash{sec}]";

        #Add the passed in message to the timestamp
        $log_string = "$cur_time $log_string\n";


        #Add the message to the console
        print $log_string;

        #Add the message to runtime log
        print LOG_FILE_H $log_string;

        close LOG_FILE_H;

        open DUMMY_LOG_FILE_H, ">>$dummyRunLog"
           or die "Could not open run log file" . $dummyRunLog . "\n";
        print DUMMY_LOG_FILE_H ".\n";  
        close DUMMY_LOG_FILE_H;
    }
    else
    {
	$message_queue .= $log_string;
    }

  }	#end RunLogIt

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : LogMessage
#------------------------------------------------------------------------------------------
# This function is used to add a messages to a log file
# Arguments:
# The first argument is the file the message should be logged to
# The remaining arguments are appended to the log file, an lf is added after argument
#------------------------------------------------------------------------------------------
# mjb - 03/09/29 -   created
#------------------------------------------------------------------------------------------
#
sub LogMessage
  {
    #Assigning passed in parameters
    my ($logFile, @strings) = @_;

    #Local Variables
    my ($string);
	
    #Open the desired file and add the passed in string to it.
    open(LOGFILE, ">>$logFile") || die "problem opening $logFile";
    foreach $string (@strings) {
      print LOGFILE "$string\n"; 
    }
    close(LOGFILE);

  }#end LogMessage

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : HiliteLogMessage
#------------------------------------------------------------------------------------------
# This function is used to add a command messages to a log file
# The command message is preceeded by a line of stars
# and followed by line of dashes
#
# Arguments:
# The first argument is the file the message should be logged to
# The second argument is the command
#------------------------------------------------------------------------------------------
# mjb - 03/09/29 -   created
#------------------------------------------------------------------------------------------
sub HiliteLogMessage
  {
    #Assigning passed in parameters
    my ($logFile, @strings) = @_;

    #Log the message with stars preceding and dashes following it
    &LogMessage($logFile, $lineofstars, @strings, $lineofdashes);
  }#end HiliteLogMessage


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CaptureLogs
#------------------------------------------------------------------------------------------
# This function invokes a set of commands and gathers the output into log.
# That log is then added (if necessary) to the appropriate list of files to be zipped
#------------------------------------------------------------------------------------------
sub CaptureLogs()
  {
    #Assigning passed in parameters
    my ($zipIdentifier, $fileName, $program, $args) = @_;

    #Local Variables
    my ($returnVal);

    #Log each of the commands that will be run
    #Then run the commmand
    &HiliteLogMessage($fileName, "$program $args");
    if($program =~ m/naviseccli/i){
      $returnVal = &InvokeCommand($program, $args, $fileName,
                                  \&CheckNaviReady,$zipIdentifier);
    }
    else
    {
      $returnVal = &InvokeCommand($program, $args, $fileName,
                                  \&EmptyStub,$zipIdentifier);
    }
    #Add one more line of dashes to the log file
    &LogMessage($fileName, $lineofdashes);
    return $returnVal;
  }	#end CaptureLogs()

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CheckValidBinary
# usage : CheckValidBinary($file)
# parameters : $file  - File to check
# return:  1 - File is a valid executable
#         -1 - File not exist
#	  -2 - File size 0
#------------------------------------------------------------------------------------------
# This function checks that a binary executable file is valid.
# NOTE
# Currently a file size of zero is checked for, but non-zero sized executables still need
# to be verified to prevent a possible process hang bug.  Possible solution may include
# some sort of hash checking (ie MD5 checksum).
#------------------------------------------------------------------------------------------
# bcm - 03/28/06 - Created
#------------------------------------------------------------------------------------------
sub CheckValidBinary($)
  {
    my ($file) = @_;
    
    if(not -e $file)
    {	
	return -1;    #the file not exist
    }
    elsif(-z $file)
    {
	return -2;   #the file size 0
    }

    return 1;
  }#end CheckValidBinary()

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : InvokeCommand
# usage : InvokeCommand($command_string, $log_file)
# parameters : $command      - command to execute
#              $command_args - command arguments
#              $log_file     - log file to append
#              $preInvokeSub - subroutine to be invoked before the actual command is run
#              $zipName - Name of the zip file into which the output of this command goes
#------------------------------------------------------------------------------------------
# This function is used to run a system command.  It should be used instead of directly
# invoking a system command so that all commands can be logged to the run log and 
# status can be checked.
#------------------------------------------------------------------------------------------
# wmb - 5/19 - Created
# bcm - 01/31/06 - Use system, redirect STDOUT to temp file, copy tempfile to log file
# bcm - 03/28/06 - Added zero file size check to prevent hung processes from an invisible
#                  Win32 dialog when a executable is not valid, non-zero but "valid" binary
#                  files are still a possible problem.
# bcm - 03/29/06 - Arguments passed in seperate from executable
#------------------------------------------------------------------------------------------
sub InvokeCommand($$$$$)
  {
    #Assigning passed in parameters
    my($command, $command_args, $log_file, $preInvokeSub,$zipName ) = @_;

    #Handle executable file error
    if(CheckValidBinary($command) == -1)
    {
	&RunLogIt("Error: Executable file not exist ($command)");
	return -1;
    }
    elsif(CheckValidBinary($command) == -2)
    {
	&RunLogIt("Error: Executable file size 0 ($command)");
	return -1;
    }

   return InvokeCommand_($command, $command_args, $log_file, $preInvokeSub, $zipName);
  }				#end InvokeCommand

sub InvokeCommand_($$$$$)
  {
    #Assigning passed in parameters
    my($command, $command_args, $log_file, $preInvokeSub, $zipName) = @_;
    
    if (ref($preInvokeSub) eq 'CODE') {
        &$preInvokeSub();
    } else {
        print STDERR "Reference to preInvokeSub is bad\n";
    }
   
    #Local Variables
    my($system_return, $log_string, $tempFile, $command_string,$exitcode, $ProcessObj,$start,$elapsed);

    $command_string = $command." ".$command_args;

    #Assemble a message stating the command to be run
    $log_string = "Invoking Command: [$command_string]";

    #Add the command message to the runtime log
    &RunLogIt($log_string);

    # Redirect STDOUT to temp file
    $tempFile = GetTempFilename();

    # Create STDOUT File handle
    open(STDOUTF, ">$tempFile");
    
        #First, duplicate the old STDERR stream so that we can restore it later
    open (OLDSTDOUT, ">&STDOUT") or die "Couldn't dup STDOUT: $!";

    # Create STDOUT File handle
    open(STDOUT, ">$tempFile");

    $exitcode = -1;
    
    sub ErrorReport{
                print STDERR Win32::FormatMessage( Win32::GetLastError() );
        }
	#using system is a work around so to avoid create failure in win2k8
	system "$command $command_args";

    #Win32::Process::Create($ProcessObj,
    #                            $command,
    #                            "$command $command_args",
    #                            0,
    #                            NORMAL_PRIORITY_CLASS,
    #                            ".")|| die ErrorReport();
    #$start = time;                                
    #$ProcessObj->Wait($defaultTimeout);
    #$elapsed = time - $start;

    #if($elapsed >= $defaultTimeout/1000) {
    #    print STDERR "Waiteded for 25 minutes so kill the process \n";
    #  $ProcessObj->Kill( $exitcode );
    #}

    $? = 0;
    close(STDOUTF);
    
    #Reconnect the stream
    open (STDOUT, ">&OLDSTDOUT") or die "Can't restore STDOUT: $!";
    #Close the old file handle
    close OLDSTDOUT;

    
    # Copy temp file to log_file
    if ( not($log_file eq "") ) {
      CopyFileInto($tempFile, $log_file);
    }

    
    # Make entry in the intermediate inventory file.
    if ( not($log_file eq "") ) {
      &InsertInventoryFileEntry ($zipName,$log_file,$tempFile);
    }

    #  If the status of the command just run is not 0,
    #  then add the status returned to the runtime log.
    if ($? != $RETURN_GOOD) {
      $log_string = "Warning: Return Value was: $?\n";
      &RunLogIt($log_string);
    }

    #Return the status of the command that was run
    return $?;
  }
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GenLogName
#------------------------------------------------------------------------------------------
# This function generates an sp specific log name from a key
#  Argument 1 is the key.
#------------------------------------------------------------------------------------------
sub GenLogName
  {
    return $operatingHash{sp_id} . "_" . $_[0];
  }				#GenLogName


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GenUniqueLogName
#------------------------------------------------------------------------------------------
# This function generates an sp specific and unique log name from a key
#  Argument 1 is the key.
#------------------------------------------------------------------------------------------
sub GenUniqueLogName
  {
    return $operatingHash{sp_serial}  . "_" . $operatingHash{sp_id} . "_" . 
           $operatingHash{file_date}.  "_" . $_[0];
  }#end GenUniqueLogName
  
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : LabelLog
#------------------------------------------------------------------------------------------
# Usage: &LabelLog($group, $file, @notes)
# Purpose: This function is called by the ini file to put a not in a file that is being
# generated. The function will create the file if this is to be the first entry into it.
# Parameters: $group - The restricted execution groups that this function call belongs to
#             $file - The name of the file to be created or appended to
#             @notes - An array of strings to be appended to the file
#------------------------------------------------------------------------------------------
# dad - 11/04 - Created
#------------------------------------------------------------------------------------------
sub LabelLog($$@)
  {
    #local variables
    my($group, $file, @notes) = @_;

    if (CheckGroup($group)) {
      #create the fully qualified name for the file
      $file = &GenLogName($file);

      if (!(-e $file)) {
	#adding the log identification string
	 &LogMessage($file, $operatingHash{log_identifier}, $lineofstars); 
      }

      #Call the function responsible for the logging
      &HiliteLogMessage($file, @notes);
    }
  }#end LabelLog

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GiveStatus
#------------------------------------------------------------------------------------------
# Usage: &GiveStatus($group, @status_strings);
# Purpose: This function will send the passed in strings to the stdout log file, as well as
#          to STDOUT itself. This function is used whenever status about what spcollect is 
#          doing must be given.
# Parameters: $group - The restricted execution groups that this function call belongs to
#             @status_strings - The messages to send to stdout log
#------------------------------------------------------------------------------------------
# dad - 11/04 - created
#------------------------------------------------------------------------------------------
sub GiveStatus($@)
  {
    #local variables
    my($temp_msg);

    #Setting passed in vars
    my($group, @status_strings) = @_;

    if (CheckGroup($group)) {
      #Opening the stdout log file for appending
      open STDOUT_FILE, ">>" . $operatingHash{stdo_file}
	    or die "Could not open stdout file" . $operatingHash{stdo_file} . "\n";

      #posting each of the messages and then adding one extra newline for neatness
      foreach $temp_msg (@status_strings) {
	    print STDOUT_FILE "$temp_msg\n";
	    print STDOUT "$temp_msg\n";
      }
      print STDOUT_FILE "\n";
      print STDOUT "\n";
      close STDOUT_FILE;
    }
    return $TRUE;
  }	#end GiveStatus

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : CheckGroup
#------------------------------------------------------------------------------------------
# Usage: &CheckGroup($group);
# Purpose: This function is designed to filter out function calls when a reduced spcollect
# is called for. A reduced spcollect is specified by command line flags, and then those
# flags are compared for each function call to the group memberships listed in the $group 
# variable. The function returns $TRUE if this function call is supposed to run now, and
# returns false if it not supposed to run now.
# Parameters: $group - The restricted execution groups that this function call belongs to
#------------------------------------------------------------------------------------------
# dad - 12/04 - created
#------------------------------------------------------------------------------------------
sub CheckGroup($) {
  #setting passed in vars
  my($group) = @_;

  #local variabls
  my($to_return, $argument);

  # override, which ensures that zip files will always be created
  if($group eq "always")
   { return $TRUE; }


  #Determine whether this function call is supposed to run now by checking to see if:
  #There were no restrictive arguments passed to spcollect or
  #That $groups contains at least one of the arguments passed to spcollect
  if( !defined $argHash{'mode'} || $group =~ m/$argHash{'mode'}/ ) {
    $to_return = $TRUE;
  } else {
    $to_return = $FALSE;
  }

  return $to_return;
}	#end CheckGroup	

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : CheckLayeredApp
#------------------------------------------------------------------------------------------
# Usage: &CheckGroup($program);
# Purpose: This function checks whether layered app information is supposed to be gathered
# for a given command, and if so checks to see if the layered app is enabled. This function
# returns an array with two members. The entries are as follows:
#
# The first entry in the array returns $TRUE if a layered app is called for and is enabled 
# or if a layered app is not called for at all. It returns $FALSE if a layered app is 
# called for and is not enabled.
#
# The second entry in the array is a message string to be used for logging.
#
# Parameters: $program - This is the value to check to see if it calls for a layered app to
# be enabled. Currently this value can be one of three things:
#     -The first argument passed to a NaviSeccli call (which may require a layered app)
#     -"mvacli" for a call to mvacli (which obviously requires mva to be enabled)
#------------------------------------------------------------------------------------------
# dad - 01/05 - created
# AR455185 - Remove the check for SnapView and SanCopy
#------------------------------------------------------------------------------------------
sub CheckLayeredApp($) {
  #setting passed in vars
  my($program) = @_;

  #create local vars
  my ($proceed, $message, $package, $key);
  ($package, $key) = split(/ /, $program);
  $proceed = $FALSE;
  $message = "Can't run  $program \n $package is NOT an enabled package";

  if ($program =~ /mirrorview|mirror/i) {
    if( $key =~ /-async/i) {
      $message = "Can't run  $program \n $package -async is NOT an enabled package";
      if ($ENV{'MVAOPTION_REVISION'}) {
        $proceed = $TRUE;
        $message = "Can run $program \n $package -async is an enabled package";
      }
    }
    else {
      if ($ENV{'MIRRORVIEWOPTION_REVISION'}) {
        $proceed = $TRUE;
        $message = "Can run $program \n $package is an enabled package";
      }
    }
  } elsif ($program =~ /mvacli/i) {
    if ($ENV{'MVAOPTION_REVISION'}) {
      $proceed = $TRUE;
      $message = "Can run $program \n $package is an enabled package";
    }
  } else {
    #we assume that the command is not requesting information from a layered app
    $proceed = $TRUE;
    $message = "The Layered app $program is enabled, Proceed with the function call.";
  }
  return ($proceed, $message);

    } #end CheckLayerdApp
    
###########################################################################################
##########                             End Subroutines                           ##########
###########################################################################################

###########################################################################################
##########                               main                                    ##########
###########################################################################################

#print the revision of spcollect
print "\nspcollect.pl revision $revision\n";

#Do some setup and checking before we start the real work
&PreRunSetup(); 

#Gather all data from the array that we want to send back
&GatherAllData();

# Final activity: list to stdout all zip files that live in c:\dumps
#&ListFiles();

exit;

###########################################################################################
##########                             End Program                               ##########
###########################################################################################
    
