###############################################################################
# This script creates a zipped up collection of files. This collection is the 
# SPCollect data.zip file. 
# This script has been refactored from spcollect_backend.pl
# The top level working directory is also passed in as an argument. 
# This script works on the directory structure created by the data gather script.
# It expects the following dir structure:-
# c:\dumps\<top_level_dir>\<inifile_name1>\,
# c:\dumps\<top_level_dir>\<inifile_name2>\
# .
# .

use Getopt::Long;
use POSIX qw(strftime);
use Win32::Service;
use Win32::EventLog;
use Win32::TieRegistry( Delimiter=>"/", ArrayValues=>0 );
use Win32;
use File::Path;
# AR 592768: The File::Glob is causing Perl crashes. Switched to using the core function.
#use File::Glob  ':glob';
use strict;

#Predefined global values
my (
    $RETURN_GOOD,		#Global function return condition for success
    $RETURN_BAD,		#Global function return condition for failure
    $TRUE,			#Global true condition
    $FALSE,			#Global false condition
    $dumpdir,			#Directory where the collect zip will be saved
    $scriptname,		#The name presented to the system
    $revision,			#Current version presented to the system
    $tempdir,			#Directory where some known log files are held
    $minidumpdir,       #Directory where mini dumps are held
    $tempFileCount,    #Temporary File Counter

    # for use in putting headers into the output files generated by spcollect.
    $lineofstars,
    $lineofdashes,

    # Default program execution timeout
    $defaultTimeout,
    $spState,

    #hold the log message that generates before the log name
    $message_queue
    
   );

$RETURN_GOOD = 0;
$RETURN_BAD = -1;
$TRUE = 1;
$FALSE = 0;
$dumpdir = "C:\\dumps";
$scriptname = "spcollect_package.pl";
$revision = $ENV{'BASE_REVISION'};
$tempdir = "c:\\TEMP";
$minidumpdir = "C:\\WINNT\\Minidump";
$tempFileCount = 0;
$lineofstars = "********************************************************************************";
$lineofdashes = "--------------------------------------------------------------------------------";
$defaultTimeout = 1500;  #  Timeout value in seconds
$message_queue = "";
$spState = "";

#Global Variables
my (
    %argHash,			# The arguments passed to the script
    %operatingHash,		# Commonly used paths and globals in the script
    %zipToFilesHash, # Key is the zip file name, Value is array of file names in zip.
    %zipToCopyFilesHash, # Key is the zip file name, Value is the array of files to be copied
    @data_gather_sub_dirs, # Array of subdirectories from which to zip up relative to the working dir.
    @fileList, # Flattened list of all files in the collect.
    $collectName # name of the spcollect file.
   );

#prototypes
sub processCommandLine();
sub Usage();
#subroutine to get array serial number
sub GetArraySerialNum();
#subroutine to get SP signature
sub GetSpSignature();
sub SetUpZipHashes();
sub InitValues();
sub CreateSubZips();
sub CopyFileInto($$);
sub CleanUp();
sub CreateCollection();
sub CleanUpOldRuns();
sub SetZipIdentifierWorkingDir($);
sub ProcessIniFile();
sub GenUniqueLogName ;
sub GenLogName ;
#copy an existing file to one of the zips
sub CopyFile($$$);
# Copy existing files to one of the zips
sub CopyFiles($$$$);
#called to verify whether a specific function call should run during restricted execution
sub CheckGroup($);
#called to send output stating the current status of the collect process
sub GiveStatus($@);
# Copy existing file with spaces in the path, to one of the zips.
sub CopySpaceFile($$$$);
#Create top level log containing version and timestamp info
sub GatherVersionInfo ;
#Collect peerwatch log, spcollect log 
sub CollectPeerwatchAndSPCollectLog();
#Getting cimom hanginfo files
sub GetNaviMngrServerData();
#Getting NT Eventlogs
sub GetNtEventLogs ;
#Getting dump analysis files
sub GetAnalysisFiles ;
#Getting mini dump files
sub GetMiniDumps ;
sub FilenameDT ;
sub FormatTimeHash($);
sub GatherRtpFiles ;
sub GetKtDumpFiles ;
sub GatherFbiFiles ;
sub CaptureNorthstarLogs($$);
sub CaptureLccStats($$);
sub GetGMTime ;
sub ListFiles();
# Gather all the RecoverPoint Splitter logs under the RP zip file
sub GatherRecoverPointLogs();
sub GetFslDump();
sub GetKtLog();
sub ZipKtLog;
sub RunLogIt;
sub RunCommand;

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : RunCommand
#------------------------------------------------------------------------------------------
# This function provides a interface to execute command on the system.
#
# Arguments:
#     $cmd     -- command
#     $filter  -- pattern to filter command output
# Return value:    
#     $rv      -- exit code
#     @output  -- array of command outputs
#------------------------------------------------------------------------------------------
sub RunCommand 
{
  my ($cmd, $filter) = @_;
  my @output;

  #filter is an optional argument
  &RunLogIt("Missing input in RunCommand"),
    return
    if ( @_ < 1 );

    undef $?;
    open CMD, " $cmd 2>&1 | "
      or &RunLogIt("System cannot run command: $cmd"), return;
    undef @output;

    # Append command info on log
    &RunLogIt("RunCommand");
    &RunLogIt("command: $cmd");
    &RunLogIt("filter: $filter") if defined($filter);

    &RunLogIt("output:");
    while (<CMD>) {

        # trim empty lines
        next if /^\s*$/;

        # Append command info on debug log also
        &RunLogIt("$_");

        if ( defined($filter) ) {
            next unless /$filter/;
        }

        push( @output, $_ );
    }

    close CMD;
    my $exit_value     = $? >> 8;
    my $signals_caught = $? & 127;
    my $CORE_DUMPED    = $? & 128;

    # Append command info on debug log also
    &RunLogIt(
        "exit value: $exit_value, signal caught: $signals_caught, core dumped: $CORE_DUMPED");
    return ( $exit_value, @output );
}
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : RunLogIt
#------------------------------------------------------------------------------------------
# This function accepts a string to log to the log file. It will add a time stamp to the 
# entry also.
#
# Arguments:
#     The first argument shoud be the string to log.
#------------------------------------------------------------------------------------------
# Yujie - 24/06/2013 - Created
#------------------------------------------------------------------------------------------
sub RunLogIt
{
  my $log_string = $_[0];

  if(exists $operatingHash{log_file} && -e $operatingHash{working_dir}."\\".$operatingHash{log_file})
  {
    open (my $log_file_h, ">>".$operatingHash{working_dir}."\\".$operatingHash{log_file}) or die "Could not open run log file";

    if($message_queue)
    {
      print $log_file_h $message_queue;
      $message_queue = "";
    }

    my (%timehash, $cur_time);
   
    %timehash = GetGMTime();
    FormatTimeHash(\%timehash);

    $cur_time = "[$timehash{hour}:$timehash{min}:$timehash{sec}]";
    $log_string = "$cur_time $log_string\n";

    print $log_string;
    print $log_file_h $log_string;

    close $log_file_h or die "Could not close run log file.";
 
 }
 else
 {
   $message_queue .= $log_string;
 }
}


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : ZipKtLog 
#------------------------------------------------------------------------------------------
# This function will invoke the zipKtLogs.pl script to create the zipped ktlog file.
#
# Arguments:
#     No argument.
#------------------------------------------------------------------------------------------
# Yujie - 24/06/2013 - Created
#------------------------------------------------------------------------------------------
sub ZipKtLog
{
    my $cmd = "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\zipKtLogs.pl";

    if(not -e $cmd)
    {
      &RunLogIt("Could not find zipKtLogs.pl");
    }
    else
    {
      my $output = `$cmd 2>&1`;
      if($? >> 8 == 0)
      {
        &RunLogIt($output);
      }
      else
      {
        &RunLogIt("Failed to execute zipKtLogs.pl");
      }
      
    }
}



#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GetSpState
#------------------------------------------------------------------------------------------
# Determine the current state of this SP.
#
# Returns the state or UNKNOWN if the state cannot be determined.
# We currently expect a state of either "normal" or "degraded".
#------------------------------------------------------------------------------------------
# Command output looks like:
#
#    Peer SP is physically INSERTED
#    
#    This SP is in NORMAL MODE (not degraded). OR This SP is in DEGRADED MODE
#    Current reboot count: 0
#    
#    This SP is set to start HFON.
#    
#    This SP is set to start MCR.
#    
#    38 normal reboot(s) since the last panic
#    
#   Prior SP panics: (newest to oldest; 5 max)
#
# We capture the state from the line: "This SP is in NORMAL MODE..."
#------------------------------------------------------------------------------------------
sub GetSpState
  { 
    my $sp_state = "DEGRADED"; # If we cannot determine the state, default to DEGRADED
    
    # Get the current state of this SP
    #my $cmd_output = `$programHash{"spstate"}{"name"}`;
    my $commandName = "C:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\spstate.exe";
    my $cmd_output = `$commandName`;
    my $exit_code = $? >> 8;
    
    if ( $exit_code == 0 ) {
    
        if( $cmd_output =~ /This SP is in (\w+) MODE/ ){
            $sp_state = $1;
        }
        else{
            
            $message_queue = "ERROR - Could not determin SP state. Defaulting to $sp_state. Could not parse result: $cmd_output";
        }
    }
    else{
        
        $message_queue = "ERROR - Could not determin SP state. Defaulting to $sp_state. Command failed: ($exit_code)$!";
    }
    
    return $sp_state;
  }



#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : ListFiles
#------------------------------------------------------------------------------------------
# Print out all of the .zip files that reside in the dump directory
#------------------------------------------------------------------------------------------
# wmb - 3/03 - Put all of this code together in its own sub
#------------------------------------------------------------------------------------------
sub ListFiles()
{
	#Local Variables
	my (@zipfiles, $file);

	#Open the directory containing the collects and create a list of all of them
	opendir DUMP, "$dumpdir" or die "Unable to open dump directory!";
	@zipfiles = grep{/\.zip$/}readdir DUMP;
	closedir DUMP;

	#Output that the script has now completed building the latest collect
	print "\nEach dump has been collected and zip'ed into its own zip file.\n";
	print "All other data has been collected and zip'ed into a separate zip file.\n";
	print "Gather all of these .zip files and send to EMC-Clariion Support\n";
	print "(These files all live in $dumpdir)\n\n";

	#Output the name of each collect in the directory
	foreach $file (@zipfiles) {
		print "    $file\n\n";
	}

	print "SPCollect has now completed.\nIf you do not see the command prompt please press enter to continue...\n";

}#end ListFiles


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GetKtLog
#------------------------------------------------------------------------------------------
# get the latest two ktlog files
# ktlog file name format: ktlog--yyyymmdd-hhmmss-SP<A/B/X>.txt
# The "SPX" is for the first ktlog file the system creates when it first boots up and 
# doesn't yet know whether it is SPA or SPB.
#------------------------------------------------------------------------------------------
# xyj - 11/12 - Created
# Chl - 8/16/2013 - Modify the file name format from  ktlog--yyyymmdd-hhmmss-SP<A/B>.txt
#                   into  ktlog--yyyymmdd-hhmmss-SP<A/B/X>.txt
#------------------------------------------------------------------------------------------
sub GetKtLog()
{
	#the maximum number of ktlog files to be collected
	my $MAX_KTLOG_FILES = 2;
	if(! opendir (DUMP, $dumpdir))
	{
		&GiveStatus("", "unable to open dump dir");
		return $FALSE;
	}
	
	my @ktlog_files = grep /^ktlog--\d{8}-\d{6}-SP[A|B|X]\.txt$/i, readdir DUMP;
	closedir DUMP;

	if( @ktlog_files == 0)
	{
		#there is no ktlog files
		&GiveStatus("", "there is no ktlog files");
		return $FALSE;
	}
	elsif( @ktlog_files <= $MAX_KTLOG_FILES )
	{
		#there are at most two ktlog files
		foreach (@ktlog_files)
		{
			push(@{$zipToCopyFilesHash{"sus"}}, $dumpdir."\\".$_);
		}
		return $TRUE;
	}
	else
	{
		#there are more than two ktlog files
		my @sorted_file = sort { -C $dumpdir."\\".$a <=> -C $dumpdir."\\".$b } @ktlog_files;
		for(my $i = 0; $i < 2; $i++)
		{
			push(@{$zipToCopyFilesHash{"sus"}}, $dumpdir."\\".$sorted_file[$i]);
		}
		return $TRUE;
	}
}



#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GetGMTime
#------------------------------------------------------------------------------------------
# Returns a hash containing time and date from gmtime
# function.
#------------------------------------------------------------------------------------------
sub GetGMTime
  {
    my %gmtimeHash;
    @gmtimeHash{"sec", "min", "hour", "day", "mon", "year", "wday", "yday"} = gmtime;

    # Adjust the year value
    $gmtimeHash{year} += 1900;

    # Adjust the month value
    $gmtimeHash{mon}++;

    return %gmtimeHash;
  }       # end GetGMTime


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GatherFbiFiles
#------------------------------------------------------------------------------------------
# Get the rls output files from fbi
#------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------
# kcd - 8/04 - created
#------------------------------------------------------------------------------------------
sub GatherFbiFiles
{
    my ($filename, @outfiles, $file, @tempfiles);

    ## configuration information file ##
    #Assemble the full name for the config info file
    $filename = &GenLogName( "fbi_info.txt");

    # Put a title line in the file...
	if(!(-e $filename)){
		#adding the log identification string
		&LogMessage($filename, $operatingHash{log_identifier});
	}
	&HiliteLogMessage($filename, "fbi info");

    #add the FBI files to the appropriate list of files to be zipped
    push (@{$zipToFilesHash{"fbi"}},$filename);

    #
    # Get the *rls* files in c:\dump
    #

    &GiveStatus("", "Status: Gather c:\\dump\\*rls* files");

    opendir DUMP, "$dumpdir" or die "Unable to open dump directory!";
    @tempfiles = readdir DUMP;
    closedir DUMP;

    # for neatness limit the matches to rls* and *.rls, case insensitive

    foreach $file (@tempfiles) {
      if ($file =~ m/(\Arls.*)|(.*\.rls\Z)/i) {
	      push @outfiles, $file;
      }
    }

    foreach $file (@outfiles) {
      #add the FBI file to the appropriate list of files to be zipped
      push (@{$zipToCopyFilesHash{"fbi"}},$dumpdir . "\\" . $file);
    }
}#end GatherFbiInfo

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : CaptureNorthstarLogs
#------------------------------------------------------------------------------------------------------------------------------
# This code searches c:\dump for all files that match m/^drive_log_\d{2}_\d{2}_\d{2}_\S*\.txt/
# All matched files are added to the sus zip and deleted
# The name format has been changed from m/^\S*_drive_log_\d{2}_\d{2}_\S*\.txt/ into m/^drive_log_\d{2}_\d{2}_\d{2}_\S*\.txt/
#------------------------------------------------------------------------------------------------------------------------------
sub CaptureNorthstarLogs($$)
{
    my ($group, $zip) = @_;
    my ($file_name, $path, @files);
    &GiveStatus("", "Status: in CaptureLogs");

    if (CheckGroup($group)) {
      opendir DUMP, "$dumpdir" or die "Unable to open dump directory!";
      @files = readdir DUMP;

      #add the files to the list of files to be zipped and deleted
      foreach $file_name (@files)
      { 
       if ($file_name =~ m/^drive_log_\d{2}_\d{2}_\d{2}_\S*\.txt$/)
         {
          $path = "$dumpdir\\$file_name";
          &GiveStatus("", "Status: gathering $path");
	  CopyFileInto($path, $file_name);
          push (@{$zipToFilesHash{$zip}},$file_name);
 	  unlink $path;
	     }
      }
	closedir DUMP;
   }
   return $TRUE;
}

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : CaptureLccStats
#------------------------------------------------------------------------------------------
# This code searches c:\dump for all files that match serialnumber_sp(a/b)_lccstats.log.
# All matched files are added to the lcc zip and deleted
#------------------------------------------------------------------------------------------
sub CaptureLccStats($$)
{
    my ($group, $zip) = @_;
    my ($file_name, $path, @files);
    
    if (CheckGroup($group)) {
      opendir DUMP, "$dumpdir" or die "Unable to open dump directory!";
      @files = grep /^.*_sp[AB]_lccstats\.log$/i, readdir DUMP;
      closedir DUMP;

      #add the files to the list of files to be zipped and deleted
      foreach $file_name (@files)
      { 
        $path = "$dumpdir\\$file_name";
        &GiveStatus("", "Status: gathering $path");
        push (@{$zipToCopyFilesHash{$zip}},$path);
      }
   }
   return $TRUE;
}# End of CaptureLccStats($$)


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GetKtDumpFiles
#------------------------------------------------------------------------------------------
# This function looks for any ktdump files that may be in the dump directory
#------------------------------------------------------------------------------------------
# wmb - 3/03 - Created
#------------------------------------------------------------------------------------------
sub GetKtDumpFiles
{
    #Local Variables
    my (@ktdfiles, $file);

    #Open the directory containing ktdump files and create a list of all of them
    opendir DUMP, "$dumpdir" or die "Unable to open dump directory!";
    @ktdfiles = grep{/ktdump.*\.txt$/i}readdir DUMP;
    closedir DUMP;

    #Check to see if any ktdump files were found
    if (@ktdfiles) {
      # Add each ktdump file to the appropriate list of files to be zipped
      foreach $file (@ktdfiles) {
        push (@{$zipToCopyFilesHash{"ktd"}},$dumpdir . "\\" . $file);
      }
    }
}#end GetKtDumpFiles


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GatherRtpFiles
#------------------------------------------------------------------------------------------
# Grab the .out files in c:\temp. (ndu folks ask for these)
#------------------------------------------------------------------------------------------
# wmb - 03/03 - Put all of this code together in its own sub
# dad - 11/04 - Moved the production of all layered app files to ini file
#------------------------------------------------------------------------------------------
sub GatherRtpFiles
{
    #Local Variables
    my ($file, @tempfiles);

    ###################################
    ## Get the .out files in c:\temp ##
    ###################################

    #Search for the .out files in C:\temp and add them to a list
    my $openTempDirResult = opendir TEMP, "$tempdir"; 
 	if($openTempDirResult){
    
	 	@tempfiles = grep{/ndu-.*\.out/i}readdir TEMP;
	    closedir TEMP;
	
	    #Traverse the list of files appending the complete path to them
	    #Add each file to the appropriate list of files to be zipped
	    foreach $file (@tempfiles) {
	      # Add each of the .out files to the list of files to be zipped
          push (@{$zipToCopyFilesHash{"rtp"}},$tempdir . "\\" . $file);
	    }
	}else{
		&GiveStatus("", "Status: " . $tempdir. " not found. Can't gather RTP files.");
		print STDERR  $tempdir . " not found.\n";
	}
}#end GatherRtpFiles


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : FormatTimeHash
# parameters : $timehash - Reference to hash created by either localtime or gmtime
#------------------------------------------------------------------------------------------
# Adjusts the contents of localtime or gmtime hash with
# leading zeros for digits below 10.
#------------------------------------------------------------------------------------------
sub FormatTimeHash($)
  {
    my ($timehash, $item);
    $timehash = $_[0];
    # adjust other single digits to have a leading zero
    foreach $item (keys %{$timehash}) {
      if (${$timehash}{$item} < 10) {
        ${$timehash}{$item} = "0" . ${$timehash}{$item};
      }
    }
  }


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : FilenameDT
#------------------------------------------------------------------------------------------
# Use the GetGMTime function to get a time and date
# stamp for the construction of the output filename.
#------------------------------------------------------------------------------------------
sub FilenameDT
  {
    #Local Variables
    my(%timehash);

    # get the time
    %timehash = GetGMTime();

   # adjust other single digits to have a leading zero
    FormatTimeHash(\%timehash);

    #Construct the final time string to return
    my $filedate =  $timehash{year}. "-" . $timehash{mon} . "-" . $timehash{day} . "_" .
      $timehash{hour} . "-" . $timehash{min} . "-" . $timehash{sec};

    return $filedate;
  }				#end FilenameDT


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GetMiniDumps
#-----------------GetMiniDumps------------------------------------------------------------------------
# This function gathers all the minidumps under the dir "C:\WINNT\Minidump". 
#------------------------------------------------------------------------------------------
sub GetMiniDumps
  {
    #Local Variables
    my ( $file, @minidumpfiles);

    if (CheckGroup ("")){#don't gather  minidump in brief mode
	    #Output that the script is now getting minidumps
	    &GiveStatus("", "Status: Gather minidumps");
	
	       
	    my $openDirResult = opendir MINIDUMPDIR, "$minidumpdir";
		if($openDirResult){
	        @minidumpfiles = grep{/Mini.*\.dmp/i}readdir MINIDUMPDIR;
		    closedir MINIDUMPDIR;
		
		    
		    #Traverse the list of files appending the complete path to them
		    #Add each file to the appropriate list of files to be zipped
		    foreach $file (@minidumpfiles) {
		      # Add each of the .dmp files to the list of files to be zipped
              push (@{$zipToCopyFilesHash{"minidump"}},$minidumpdir . "\\" . $file);
		    }
		}else{
	    	&GiveStatus("", "Status: " . $minidumpdir . " not found. Can't gather mini dumps.");
	    	print STDERR $minidumpdir . " not found.\n" ;		
		}
    }
  }	#end GetMiniDumps

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GetAnalysisFiles
#-----------------GetAnalysisFiles------------------------------------------------------------------------
# This function gathers all the analysis files under the dir "C:\Dumps". 
#------------------------------------------------------------------------------------------
# mwh - 7/05 - Created
#------------------------------------------------------------------------------------------
sub GetAnalysisFiles
  {
    #Local Variables
    my ( $file, @analysisfiles);

    if (CheckGroup ("")){#don't gather  minidump in brief mode
        #Output that the script is now getting analysis files
        &GiveStatus("", "Status: Gather dump analysis files");
            
        my $openDirResult = opendir DUMPDIR, "$dumpdir";
        if($openDirResult){
            @analysisfiles = grep {/^SP[\w\W]*.analysis.txt$/} readdir DUMPDIR;
            closedir DUMPDIR;
            
            #Traverse the list of files appending the complete path to them
            #Add each file to the appropriate list of files to be zipped
            foreach $file (@analysisfiles) {
                # Add each of the .dmp files to the list of files to be zipped
              push (@{$zipToCopyFilesHash{"dumpanalysis"}},$dumpdir . "\\" . $file);
            }
        }else{
            &GiveStatus("", "Status: " . $dumpdir . " not found. Can't gather dump analysis files.");
            print STDERR $dumpdir . " not found.\n" ;       
        }
     }
  }	#end GetAnalysisFiles


sub CollectPeerwatchAndSPCollectLog()
{
    $operatingHash{spcollect_log_file} = &GenLogName("SPCollect_log.txt");    
    CopyFileInto("C:\\dumps\\SPCollect_log.txt", $operatingHash{spcollect_log_file});
    push (@{$zipToCopyFilesHash{"top"}},$operatingHash{spcollect_log_file});
     
    $operatingHash{peerwatch_log_file} = &GenLogName("PeerWatchLog.txt");
    CopyFileInto("C:\\dumps\\PeerWatchLog.txt", $operatingHash{peerwatch_log_file});
    push (@{$zipToCopyFilesHash{"top"}},$operatingHash{peerwatch_log_file});

}

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : HiliteLogMessage
#------------------------------------------------------------------------------------------
# This function is used to add a command messages to a log file
# The command message is preceeded by a line of stars
# and followed by line of dashes
#
# Arguments:
# The first argument is the file the message should be logged to
# The second argument is the command
#------------------------------------------------------------------------------------------
# mjb - 03/09/29 -   created
#------------------------------------------------------------------------------------------
sub HiliteLogMessage
{
    #Assigning passed in parameters
    my ($logFile, @strings) = @_;

    #Log the message with stars preceding and dashes following it
    &LogMessage($logFile, $lineofstars, @strings, $lineofdashes);
}#end HiliteLogMessage

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : LogMessage
#------------------------------------------------------------------------------------------
# This function is used to add a messages to a log file
# Arguments:
# The first argument is the file the message should be logged to
# The remaining arguments are appended to the log file, an lf is added after argument
#------------------------------------------------------------------------------------------
# mjb - 03/09/29 -   created
#------------------------------------------------------------------------------------------
#
sub LogMessage
{
    #Assigning passed in parameters
    my ($logFile, @strings) = @_;

    #Local Variables
    my ($string);
	
    #Open the desired file and add the passed in string to it.
    open(LOGFILE, ">>$logFile") || die "problem opening $logFile";
    foreach $string (@strings) {
      print LOGFILE "$string\n"; 
    }
    close(LOGFILE);

}#end LogMessage
  
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GatherVersionInfo
#------------------------------------------------------------------------------------------
# Figure out which version and dumpdir we are using, and put that in our records.
#------------------------------------------------------------------------------------------
# wmb - 3/03 - Put all of this code together in its own sub
#------------------------------------------------------------------------------------------
sub GatherVersionInfo
  {
    #local variable
    my($file_name);

    #create a unique name for the log
    $file_name = &GenLogName("collect_name_rev.txt");
	
    #Create an entry for the version info file in the zip Hash
    $operatingHash{fname_name_rev} = $file_name;

    #Add header and timestamp to the version info file
    &HiliteLogMessage($operatingHash{fname_name_rev}, 
		      "Name of collect script: $scriptname",
		      "Version of collect script: $revision");
    &LogMessage($operatingHash{fname_name_rev}, 
		"Start time of operation: " . $operatingHash{file_date});
    #Add this file to the zip hashes to be included in the final zip file
    push (@{$zipToFilesHash{"top"}},$operatingHash{fname_name_rev});	
    
}#end GatherVersionInfo

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CopySpaceFile
# usage : CopyFile($group, $zip, $location)
# parameters : $group - The restricted execution groups that this function call belongs to
#              $zip - The nice name of the zip file where the file will be copied to.
#              $path - The path (with spaces in it) to the file to be copied.
#              $filename - The name of the file to be copied
# return values: This function will return $FALSE if the given file could not be copied for
# any reason, or returns $TRUE on a success.
#------------------------------------------------------------------------------------------
# This function will be called by the external configuration file. It will copy a file from
# the given location into the given zip file. Checks are done on the zip file to make sure
# it exists, and checks are done of the path to make sure it exists and to see if it
# contains any environment variable dependent directories.
#
# This function is to be used to copy an existing file that has a pathname that has space(s).
# This function uses a temporary file, without spaces in the pathname, to copy the original
# file into. We also prepend the SP designation to the temporary file (SPx_). This function
# calls the ziphash functionality directly, with a value of "created", since we don't want
# to keep the temporary file.
#------------------------------------------------------------------------------------------
sub CopySpaceFile($$$$)
{
    my ($location, $safefilename);

    #value to be returned
    my $to_return = $TRUE;

    #Setting locals to the passed in parameters
    my($group, $zip, $path, $filename) = @_;
    if(! $filename)
    {
	$to_return = $FALSE;
    }
    elsif(CheckGroup($group)) {

        #Replacing any environment variable names with values in the passed in path
        $path =~ s/%([^%]*)%/$ENV{$1} || "%$1%"/eg;

        # Construct teh full path to the desired file
        $location = "$path"."$filename";

        # Make sure the translation worked
        if (!(-e $location)) {
            $to_return = $FALSE;
        }

        # Now copy to a temp file, with no spaces in the path. Also prepending SPx to the safefilename
        else {
            $safefilename = &GenLogName("$filename");
            CopyFileInto($location, $safefilename);

            #Checking the existence of the file to copy and the zip file
            if (!(-e $safefilename)) {
                $to_return = $FALSE;
            }

            #Add the file to the list of existing files in the given zip
            else {
                  push (@{$zipToFilesHash{$zip}},$safefilename);
            }
        }
    }#end checkgroup

 return $to_return;
}#end CopySpaceFile



#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GetNaviMngrServerData
#------------------------------------------------------------------------------------------
# Purpose: Add cimom hang info files to the "nav" zip
#------------------------------------------------------------------------------------------
# kcd - 09/04 - Added lastNTEvtIndices.log to list
# dad - 11/04 - Stripped navi files out of function and put them into ini file
#------------------------------------------------------------------------------------------
sub GetNaviMngrServerData()
{
    #Local Variables
    my (@cimomFiles);

    #outputing to the stdout log
    &GiveStatus("", "Status: Gather cimom files from spcollect");

    #check for CIMOM_hanginfo files
    opendir DUMP, "$dumpdir" or die "Unable to open dump directory!";
    @cimomFiles = grep /^CIMOM_hanginfo.*\.txt$/i, readdir DUMP;
    closedir DUMP;

    #add the files to the list of files to be zipped
    my ($cimomLog);
    foreach $cimomLog (@cimomFiles)
    {
      push (@{$zipToCopyFilesHash{"nav"}},$cimomLog);
    }
}#end GetNaviMngrServerData

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GetFslDump
#------------------------------------------------------------------------------------------
# This function gathers the dump of Pool Service.
#------------------------------------------------------------------------------------------
sub GetFslDump()
{
	&GiveStatus("", "Status: Gathering pool service data");
	

	if(! opendir(DUMP, $tempdir))
	{
		&GiveStatus("", "temp dir can not be opened");
		return $FALSE;
	}

	#Try to match "PoolSrvcDump_dddddd.txt"
	my @pooldump = grep /^(PoolSrvcDump_)(\d+\.)txt$/i, readdir DUMP;
	foreach my $file (@pooldump)
	{
		CopyFileInto($tempdir."\\".$file, $file);
		push(@{$zipToFilesHash{"sus"}}, $file);
		unlink $tempdir."\\".$file;
	}
	closedir DUMP;
	return $TRUE;
}

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GetNtEventLogs
#------------------------------------------------------------------------------------------
# This function gathers all the NT Event logs together.
#------------------------------------------------------------------------------------------
sub GetNtEventLogs
{
    #Local Variables
    my ($myServer, $eventLog, $handle, $dest, $logFile);

    #Output that the script is now getting NT event logs
    &GiveStatus("", "Status: Gather NT Event Logs");

    #Get the name of the SP from the environment variables
    $myServer="\\\\$ENV{ComputerName}";

    ##Walk through each event log and save the log to a file that will be    ##
    ##added to the evt zip						     ##
    foreach $eventLog ("Application", "System", "Security", "Setup") {
      #Create a new event log object from which to extract the log data
      $handle=Win32::EventLog->new($eventLog, $myServer)
	      or die "Can't open $eventLog EventLog on $myServer\n";

      #Create a file to hold the event log data
      #AR 382737 changing extension to .evtx
      $logFile = &GenLogName($eventLog . ".evtx");

      #Save a backup of the event log data to the file
      $handle->Backup($logFile)
	      or warn "Could not backup the $eventLog EventLog on $myServer ($^E)\n";

      #add the file with the event log data to the list of files to be zipped
      push (@{$zipToFilesHash{"evt"}},$logFile);
    }

    # Close must be called last!!
    #
    # for unknown reasons, this above code fails when Close is called
    # within the loop.  The first pass completes, the rest fail..
    # I suspect that the problem is with the old version of perl used
    # on the systems.
    #
    $handle->Close;

}#end GetNtEventLogs


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GiveStatus
#------------------------------------------------------------------------------------------
# Usage: &GiveStatus($group, @status_strings);
# Purpose: This function will send the passed in strings to the stdout log file, as well as
#          to STDOUT itself. This function is used whenever status about what spcollect is 
#          doing must be given.
# Parameters: $group - The restricted execution groups that this function call belongs to
#             @status_strings - The messages to send to stdout log
#------------------------------------------------------------------------------------------
# dad - 11/04 - created
#------------------------------------------------------------------------------------------
sub GiveStatus($@)
{
  #local variables
  my($temp_msg);

  #Setting passed in vars
  my($group, @status_strings) = @_;

  if (CheckGroup($group)) {
    #Opening the stdout log file for appending
    open STDOUT_FILE, ">>" . $operatingHash{stdo_file}
	or die "Could not open stdout file" . $operatingHash{stdo_file} . "\n";

    #posting each of the messages and then adding one extra newline for neatness
    foreach $temp_msg (@status_strings) {
	  print STDOUT_FILE "$temp_msg\n";
	  print STDOUT "$temp_msg\n";
    }
    print STDOUT_FILE "\n";
    print STDOUT "\n";
    close STDOUT_FILE;
  }
  return $TRUE;
}#end GiveStatus


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : CheckGroup
#------------------------------------------------------------------------------------------
# Usage: &CheckGroup($group);
# Purpose: This function is designed to filter out function calls when a reduced spcollect
# is called for. A reduced spcollect is specified by command line flags, and then those
# flags are compared for each function call to the group memberships listed in the $group 
# variable. The function returns $TRUE if this function call is supposed to run now, and
# returns false if it not supposed to run now.
# Parameters: $group - The restricted execution groups that this function call belongs to
#------------------------------------------------------------------------------------------
# dad - 12/04 - created
#------------------------------------------------------------------------------------------
sub CheckGroup($) {
  #setting passed in vars
  my($group) = @_;

  #local variabls
  my($to_return, $argument);

  # override, which ensures that zip files will always be created
  if($group eq "always")
   { return $TRUE; }


  #Determine whether this function call is supposed to run now by checking to see if:
  #There were no restrictive arguments passed to spcollect or
  #That $groups contains at least one of the arguments passed to spcollect
  if( !defined $argHash{'mode'} || $group =~ m/$argHash{'mode'}/ ) {
    $to_return = $TRUE;
  } else {
    $to_return = $FALSE;
  }

  return $to_return;
}#end CheckGroup	

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : CopyFiles
# usage : CopyFiles($group, $zip, $directory, $regex)
# parameters : $group - The restricted execution groups that this function call belongs to
#              $zip - The nice name of the zip file where the file will be copied to.
#              $directory - The directory to apply the regular expression to.
#              $regex - Regular expression to match contents of directory with.
# return : 
#------------------------------------------------------------------------------------------
# Contructs an array of file paths for which the regular expression matches against the
# file name and copies these files to the specified zip.
#------------------------------------------------------------------------------------------
sub CopyFiles($$$$)
{
    my($mode, $zip, $directory, $regEx) = @_;
    my(@tempfiles, $file);
    
    #Replacing any environment variable names with values in the passed in path
    $directory =~ s/%([^%]*)%/$ENV{$1} || "%$1%"/eg;
    
    my $openDirResult = opendir TEMP, $directory;
      if ($openDirResult)
      {
      @tempfiles = grep { /$regEx/ } readdir TEMP;
      closedir TEMP;
      
      foreach $file (@tempfiles)
      {
        $file = "$directory\\$file";
        CopyFile($mode, $zip, $file);
      }
      
      return $TRUE;
    }
    
    return $FALSE;
}#end CopyFiles

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CopyFile
# usage : CopyFile($group, $zip, $location)
# parameters : $group - The restricted execution groups that this function call belongs to
#              $zip - The nice name of the zip file where the file will be copied to.
#              $location - The full path of the file to be copied.
# return values: This function will return $FALSE if the given file could not be copied for
# any reason, or returns $TRUE on a success.
#------------------------------------------------------------------------------------------
# This function will be called by the external configuration file. It will copy a file from
# the given location into the given zip file. Checks are done on the zip file to make sure
# it exists, and checks are done of the path to make sure it exists and to see if it
# contains any environment variable dependent directories.
#------------------------------------------------------------------------------------------
sub CopyFile($$$)
{
    #value to be returned
    my $to_return = $TRUE;

    #Setting locals to the passed in parameters
    my($group, $zip, $location) = @_;

    if(CheckGroup($group)) {
      #Check to see that the proper number of parameters were passed to the function
      if (@_ != 3) {
	      $to_return = $FALSE;
      } else 
      {

      #Replacing any environment variable names with values in the passed in path
      $location =~ s/%([^%]*)%/$ENV{$1} || "%$1%"/eg;
      
      #Checking the existence of the file to copy and the zip file
      if (!(-e $location)) {
          &GiveStatus("", $location." not found");
          $to_return = $FALSE;
      }
      #Add the file to the list of existing files in the given zip
      else {
          push (@{$zipToCopyFilesHash{$zip}},$location);
          $to_return = $TRUE;
      }
      }
    }
    return $to_return;
}#end CopyFile

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : ProcessIniFile
# description: This function reads the packaging ini file and executes the commands in it
#-----------------------------------------------------------------------------------------
sub ProcessIniFile()
{
    #locals
    my($file_line, @commands);
    my($path, $newPath, $dirname);

    #open the external file that contains most function calls
    open(INIFILE, $operatingHash{ini_file_path}) or
      &GiveStatus("", "The ini file could not be opened");

    #Gather all of the information from the file.
    @commands = <INIFILE>;
    close INIFILE;

    #Parse the information and extract the function calls.
    $file_line = join("", @commands);
    $file_line =~ s/\cJ|\cI|\n//g; #replace carriage return/line feed combos with a newline
    #The regex below does the following:
    # Looks for valid command calls in the external file
    # & - look for an ampersand 
    # \w+ - look for some word
    # \( - look for an open parethese
    # (?:)+? - find the regex inside the parentheses 0 or more times, but do not return it
    # .*? - look for at 0 or more non-whitespace characters (minimally)
    # \n* - look for zero or more new lines
    # \); - look for closing paren and then a semicolon
    # g - repeat this until the end of the stream
    # The regex is set to an array so that each successful find of the regex will have its
    # own discrete entry in the array.
    @commands = ($file_line =~ /&\w+\((?:.*?\n*)+?\);/g);

    #execute the function calls one at a time
    foreach $file_line (@commands) {
      if ((eval $file_line) == $FALSE) {
        #note that the line from the ini file was not executed correctly
        &GiveStatus("", "\tError: The line:", "\t$file_line", 
          "\tWas not executed successfully");
      }
    }
}

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CleanUpOldRuns
# description: This function cleans up any incomplete runs from previous invocations
#-----------------------------------------------------------------------------------------
sub CleanUpOldRuns()
{
    my (@leftOverDirs, $dumpdirContent, $leftOverDir);
  	
  	my (@leftOverRunlogs, %runloghash, $dir, $ranTime,$logName, $fulloldLogPath, $destLog,
  		$leftOverRunlog ); 

    #Get a listing of all the files and directories in the dumps directory
    $dumpdirContent = `dir $dumpdir`;

    #Extract the old working directory names from the directory listing
    @leftOverDirs = $dumpdirContent =~ /<DIR>\s*(spcollect_\S+)\n/g;
    
    #Extract the old runlog names from the directory listing
    @leftOverRunlogs = $dumpdirContent =~ /\s*(\S+runlog\S+)\s*\n/g;
    foreach $leftOverRunlog (@leftOverRunlogs)
    {
		unlink $dumpdir."\\".$leftOverRunlog;
    } 
    
    &InitValues();
    
    foreach $leftOverDir (@leftOverDirs)
    {
      #perform cleanup init
      &SetZipIdentifierWorkingDir($leftOverDir);
      
      #set up the zip hashes
      &SetUpZipHashes();

      #use the zip hashes and create the final zip file
      &CreateSubZips();

      #Create the final zip
      &CreateCollection();

      #Clean up directory
      &CleanUp();
    }
}



#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : Usage
# description: in the event that invalid command line values are passed to spcollect this
# function will display the correct command line usage framework to STDOUT
#------------------------------------------------------------------------------------------
sub Usage()
{
    #Local Variable
    my ($switchlist);

    #display the necessary usage information to the user
    $switchlist = "[-help] [-mode brief] [-checkNaviReady] [-iniFile] [-spcTimeStamp]";
    print "Usage:\n";
    print "   spcollect.pl $switchlist\n";
    print "    -help            : Display this message\n";
    print "    -mode brief      : when specified, selects subset to be captured\n";
    print "                       brief defines smallest/fastest collect mode\n";
    print "    -checkNaviReady  :When specified navi readiness is checked before navi commands are issued\n";
    print "    -iniFile <ini file for data gather> : This is the inifile that defines the data to be capured\n";
    print "    -spcTimeStamp <time stamp> : This is normally the time when SPCollect was invoked and the name of the top level directory\n";
    exit;
}#end Usage

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
#DIMS 211315
# sub : GetArraySerialNum
# description : This function uses speclcli to get the array serial number.
#------------------------------------------------------------------------------------------
sub GetArraySerialNum()
{
    my($serial_num, $line, @speclcli_resume_cmd_output, $commandName);
    $serial_num = "NOSERIALNUM";
    

    if( $ENV{'VM_SIM'} eq "1" )
    {
        my $tmp = $Registry->{"LMachine/System/CurrentControlSet/services/Flaredrv/Parameters//FlareNoResPromSerialNumber"};
        if( $tmp ne "" )
        {
            $serial_num = $tmp;
            return $serial_num;
        }
    }
    else
    {
	
        my $midplane_id = 0;

        #(1),get the id of midplane with the command "speclcli -getresume";
	#(2),get the SN with the midplane id captured in (1);
        $commandName = "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\speclcli.exe";

	my @getresume_output = `$commandName -getResume`;
	foreach my $getresume_line (@getresume_output)
	{
		#try to match for the line "xx - Midplane", such as "13 - Midplane"
		if($getresume_line =~ /(\d+)(\s*)-(\s*)midplane/i)
		{
		     $midplane_id = $1;
		     last;
		}
	}

	#write the error message into an array variable when -getResume fails
	if($midplane_id == 0)
	{
		$message_queue = "Can't find id for midplane in package phase\n";
		$message_queue .= "output of -getresume:\n";
		$message_queue .= join( "\n", @getresume_output );
		
		return $serial_num;
	}
        @speclcli_resume_cmd_output = `$commandName -getResume $midplane_id 2>&1`;
	if( ! @speclcli_resume_cmd_output )
	{
		$message_queue = "Command returns nothing!\n";
		return $serial_num;
	}

        foreach $line (@speclcli_resume_cmd_output)
        {
            #look for the line with "EMC Product Serial Number"
            if($line =~ /EMC\s+Product\s+Serial\s+Number\s+:\s+(.*)/)
            {
                my $serial_num_temp = $1;
                $serial_num_temp =~ s/\s+$//;
                if( $serial_num_temp =~ /\w{5}\d{9}/i )
		{
			$serial_num = $serial_num_temp;
		}
		else
		{
			$message_queue = "Found EMC Product Serial Number line, but the serial number is invalid!\n";
			$message_queue .= "the returned serial number is: $serial_num_temp\n";
		}
		return $serial_num;
            }
        }
	$message_queue = "Could not find EMC Product Serial Number line!\n";
	$message_queue .= "output of -getresume $midplane_id:\n";
	$message_queue .= join("\n", @speclcli_resume_cmd_output);
	return $serial_num;
    }
}

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
#DIMS 211315
# sub : GetSpSignature
# description : This function uses getSpids to get the sp's signature.
#------------------------------------------------------------------------------------------
sub GetSpSignature()
{
  my($signature, $spids_line, @getspids_cmd_output, $commandName);
  $signature = "NOSIG";
  
  $commandName = "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\getSpIds.exe";
  
  if(uc($spState) eq "NORMAL")
  {
  	@getspids_cmd_output = `$commandName`;
  
  	foreach $spids_line (@getspids_cmd_output)
  	{
    		 # "*" in the o/p indicates the current SP. Look for [00hexnumber]. This is the signature
    		 if($spids_line =~ /\*\s+[0-9|a-z]+:[0|1]\s+\[([0-9|a-z]+)/i)
     		{ 
	  		$signature = $1;
	  		$signature =~ s/^0+//;
	 		 return $signature;
     		}
  	}
  }
  else
  {
  	return $signature;
  }
}

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GetSpSignature
# description : This function reads the command line and sets up some internal data structures.
#------------------------------------------------------------------------------------------
sub ProcessCommandLine()
{
    unless (GetOptions(\%argHash, 'help', 'mode=s', 'checkNaviReady', 'spcTimeStamp=s','iniFile=s','cleanup'))
    {
	  &Usage();
    }

    #Set the timestamp that will be used throughout the script
    $operatingHash{file_date} = $argHash{spcTimeStamp};
    
}

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : SetZipIdentifierWorkingDir
# description: This function sets the working directory and zip identifier 
# in the operating hash and changes the working directory to the operatingHash working dir
#------------------------------------------------------------------------------------------
sub SetZipIdentifierWorkingDir($)
{
  my ($workingDir) = @_;
  if($argHash{'cleanup'})
  {
    $operatingHash{zip_identifier} = $operatingHash{sp_serial} . "_" . $operatingHash{sp_id} . 
                                     "_" . $operatingHash{file_date} . "_old_data";
    
    # Create the name of the SPCollect zip file:
    $collectName = $operatingHash{zip_identifier};
  }
  else
  {
    $operatingHash{zip_identifier} = $operatingHash{sp_serial} . "_" . $operatingHash{sp_id} . "_" .
                                     $operatingHash{file_date}  . "_" . $operatingHash{sp_sig} . "_";
    # Create the name of the SPCollect zip file:
    $collectName = $operatingHash{zip_identifier} . "data.zip";
  }
  
  #Directory that will be used to store temporary files
  $operatingHash{working_dir} = 
            $dumpdir . "\\" .$workingDir;

  #Set the working directory as the current directory
  chdir($operatingHash{working_dir});
}

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : InitValues
# description: This funciton will gather several pieces of SP and run specific
# data that will be used throughout the script. These values will be assigned 
# to a global hash. This function changes the working directory to the top level directory
#------------------------------------------------------------------------------------------
sub InitValues()
{
    #Local Variables
    my($cmd_output, $status);

    #System information that will be used throughout the script
    $operatingHash{base_msgbin_dir} = "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin";
    $operatingHash{mirrorview_msgbin_dir} = "c:\\EMC\\MirrorView\\$ENV{'MIRRORVIEW_REVISION'}\\msgbin";
    #ini file is the passed in argument
    $operatingHash{ini_file_path} = $operatingHash{base_msgbin_dir} . "\\" . $argHash{iniFile};
    #new perl script name
    $operatingHash{spcollect_path} = $operatingHash{base_msgbin_dir} . "\\spcollect_package.pl";

    #Get SP identification
    $cmd_output = `query_sp_id`;
    $status = $?/256;
    if ( $status == 0 ) {
      $operatingHash{sp_id} = "SPA";
    } elsif ( $status == 1 ) {
      $operatingHash{sp_id} = "SPB";
    } else {
      $operatingHash{sp_id} = "SPX";
    }

    $spState = GetSpState();
    
    ############################################################################
    ##### Grab serial number and signature                 #####################
    ############################################################################
	$operatingHash{sp_serial} = GetArraySerialNum();
	$operatingHash{sp_sig}    = GetSpSignature();
	
	##The naming of spcollect impacts  
    ##1. PeerWatch library: check PeerWatchDefs.h file 
    ##2. CAP: Check ZipProcess.java file 
    
    ## Define Identifiers, used for naming zip's, files, and header messages throughout the script ##

    $operatingHash{file_name_identifier} = $operatingHash{sp_serial}. "_" .  $operatingHash{sp_id} . "_" .
      $operatingHash{file_date} . "_" . $operatingHash{sp_sig} . "_";

    $operatingHash{log_identifier} = "#spcollect " . $operatingHash{sp_serial} . "_" . $operatingHash{sp_id} . "_" . 
       $operatingHash{file_date} . "_" . $operatingHash{sp_sig};
       
    $operatingHash{log_file} =  &GenUniqueLogName("runlog.txt");   
    $operatingHash{stdo_file} = &GenUniqueLogName("stdout_log.txt");
    $operatingHash{stde_file} = &GenUniqueLogName("stderr_log.txt");
}
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GenUniqueLogName
#------------------------------------------------------------------------------------------
# This function generates an sp specific and unique log name from a key
#  Argument 1 is the key.
#------------------------------------------------------------------------------------------
sub GenUniqueLogName
{
    return $operatingHash{sp_serial}  . "_" . $operatingHash{sp_id} . "_" . 
           $operatingHash{file_date}.  "_" . $_[0];
}#end GenUniqueLogName

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GenLogName
#------------------------------------------------------------------------------------------
# This function generates an sp specific log name from a key
#  Argument 1 is the key.
#------------------------------------------------------------------------------------------
sub GenLogName
  {
    return $operatingHash{sp_id} . "_" . $_[0];
  }				#GenLogName


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : SetUpZipHashes
# Description: This function reads the intermediate inventory files in the ini file sub directories
# and creates the appropriate hashes to be used by the fuctions that follow.
#------------------------------------------------------------------------------------------
sub SetUpZipHashes()
{
 #Local Variables
 my ($i, $j, $dirContent, @tempInvFile, $tempDirVar, 
     @interInvFilePath, $interInvFile, @entries, $entry, @fields, $subDir, $workingDirContent);
 
 $workingDirContent = `dir .`;
 #There should not be any other files in this directory. Only ini file sub directories.
 @data_gather_sub_dirs = $workingDirContent =~ /<DIR>\s*(\w\S+)\n/g;

 #@data_gather_sub_dirs = grep(/(\w\S+)/g,readdir(DIR));
 
 #print "@data_gather_sub_dirs"."\n";
 
 #look for the intermediate inventory file in the sub dirs
 $i = 0;
 foreach $subDir (@data_gather_sub_dirs)
 {
    opendir(DIR, ".\\".$subDir);
    @tempInvFile = grep(/_inventory/g,readdir(DIR));
    closedir(DIR);
    #There should be only one inv file in each sub dir hence only index 0 used.
    @interInvFilePath[$i] = $operatingHash{working_dir}."\\".$subDir."\\".@tempInvFile[0];
    #print "@interInvFilePath[$i]\n";

    $i++;
 }
 #@interInvFilePath now contains the full path of the intermediate inventory files
 #Collect all the files in the sub zip files and all the temp files in the files in the sub zip file.
 $i = 0;
 @fileList = []; # initialize flattened file array to nothing.
 foreach $interInvFile (@interInvFilePath)
 {
    $interInvFile = @interInvFilePath[$i];
    if(open(INVFILE, $interInvFile)){
		#Gather all of the information from the file.
		@entries = <INVFILE>;
		close INVFILE;
		$j = 0;
		foreach $entry (@entries)
		{
		$entry = @entries[$j];
		#@field[0] = zip file name
		#@field[1] = content file name
		#@field[3] = temp file name to be copied into content file
		@fields = split ',' , $entry; 
		if (not grep ($_ eq @fields[1], @fileList)){
			push (@fileList,@fields[1]);
			#do not add runlog, stdout and stderr files. These will be 
			#collected via filelist array since these require concatenation.
			if(not (@fields[1] =~ /runlog/ or @fields[1] =~ /stderr/ or @fields[1] =~ /stdout/)){
				push (@{$zipToFilesHash{@fields[0]}}, @fields[1]);
			}
		}
		$j++;
		}
		$i++;
    } else {
		GiveStatus("", "The intermediate inventory file could not be opened");
    }
  }
  
  my ($file, $subDirFile, $tempFile);
  #create top level files in top level directory from files in sub directories
  foreach $file (@fileList)
  {
    foreach $subDir (@data_gather_sub_dirs)
    {
      $subDirFile = ".\\".$subDir."\\".$file;
      if(-e $subDirFile)
      {
        CopyFileInto($subDirFile, $file);
      }
    }

    #write the error message of -getResume into the runlog file when the $midplane_id_log variable is not null
    if( $file =~ /runlog/ && $message_queue )
    {
 	open LOG_FILE_H, ">>$file" or die "Can't open runlog file\n";
	print LOG_FILE_H $message_queue;
	close LOG_FILE_H; 
	$message_queue = "";
    }

  }
 }

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CopyFileinTo
# parameters : $originalFile - Path to original file
#              $targetFile   - Path to target file to append to
#------------------------------------------------------------------------------------------
# Appends the contents of the file specified by the first argument to
# the file specified by the second argument.  Creates the second file
# if it does not exist.
#------------------------------------------------------------------------------------------
sub CopyFileInto($$)
  {
    my($originalFile, $targetFile) = @_;

    #Create handles to files
    open (ORGFILE, "< $originalFile")
      or print STDERR "CopyFileInto Could not open original file ($originalFile): $!\n";
    open (TGTFILE, ">> $targetFile") 
      or print STDERR "CopyFileInto Could not open target file ($targetFile): $!\n";

    binmode(ORGFILE);
    binmode(TGTFILE);

    while (<ORGFILE>) {
      print TGTFILE;
    }

    #Close handles
    close TGTFILE;
    close ORGFILE;
  } #end CopyFileInto

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GatherRecoverPointLogs
#------------------------------------------------------------------------------------------
# Usage: &GatherRecoverPointLogs;
# Purpose: This function gathers all the RecoverPoint Splitter logs under the directory
#"C:\dumps" . This function also deletes previous RecoverPoint log prior to generating
# the new one.
#
# Parameters: None
#------------------------------------------------------------------------------------------
# NS - 05/20 - created
#------------------------------------------------------------------------------------------
sub GatherRecoverPointLogs() {

   my($filename, $command, $rv, @output);
   
   #Delete old RecoverPoint log
   $command = "del " . $dumpdir . "\\" ."*_RP.zip";

   if(glob($dumpdir . "\\" ."*_RP.zip")){
    RunLogIt( "Deleting old RP archive");
    ($rv, @output) = &RunCommand($command); 
    if (!defined($rv) or ($rv != 0) ){

      RunLogIt( "WARNING: Delete command failed");

    }  
   }  
         
   #Get the new RecoverPoint log name
   $filename = &GenUniqueLogName("RP.zip");
   
   #Add Splitter logs and tweak files to the RP zip
   &GiveStatus("", "Status: Gather RecoverPoint Splitter logs");
   
   #AR 591172 check whether environment variable TWEAK_PATH is defined. 
   #If TWEAK_PATH is undefined, the files under TWEAK_PATH will not be gathered.
   my $prefix = "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\zip.exe" . " -j -q -g " . $dumpdir . "\\" 
                . $filename . " " . $dumpdir . "\\" . "rpsplitter.log.*";

   if (defined($ENV{TWEAK_PATH}) && $ENV{TWEAK_PATH} ne "") {
      $command = $prefix . " " . $ENV{TWEAK_PATH} . "\\" . "*" ;        
    }
    else {
      $command = $prefix;
      RunLogIt( "The environment value TWEAK_PATH is undefined");
    }

   ($rv, @output) = &RunCommand($command); 
   #$result = `$command`;
   #Record the result of zip command into the log file.
   if (!defined($rv) or ($rv != 0) ) {
      RunLogIt( "WARNING: Zip command failed");    
   }
   else{
      RunLogIt( "Zip command succeeded");     
   }
   
} #end GatherRecoverPointLogs


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CreateSubZips
# Description: This function processes the hash file created in the previous step and creates the
# sub zip files in the top level directory
#------------------------------------------------------------------------------------------
sub CreateSubZips()
{
 my ($zipFileName, $zipName, $fileList, $item, $zipCommand,$copyCommand,$inv_file);
 #Iterate over each zip file in the zip files hash and create the zip file with contents
 #specified in value array.
 $fileList = "";
 #create a filename for, label, and open the inventory file
 $inv_file = &GenUniqueLogName("zip_inventory.txt");
 &HiliteLogMessage($inv_file, "Inventory of " . $collectName);
 &LogMessage($inv_file, "\n" . $collectName,
		"\t" . $operatingHash{stdo_file},
		"\t" . $operatingHash{stde_file},
	    "\t" . $operatingHash{log_file}, 
	    "\t" . $operatingHash{fname_name_rev}, 
	    "\t" . $operatingHash{spcollect_log_file}, 
	    "\t" . $operatingHash{peerwatch_log_file},
	    "\t" . $inv_file);
 #add the inventory file to be collected	    
 push (@{$zipToFilesHash{"top"}},$inv_file);
 foreach $zipName (keys %zipToFilesHash) 
 {
    if(not $zipName =~ /top/)
    {
      $zipFileName = $operatingHash{zip_identifier}.$zipName;
	  #Add key to the inventory file
	  &LogMessage($inv_file, "\t" . $zipFileName);
      $fileList = "";
      foreach $item (@{$zipToFilesHash{$zipName}})
      {
			if($zipName =~ /lcc/)
			{
			    #if the sub zip is lcc then the tool creates a zip file that is
			    #the name of the final sub zip file so instead of trying to zip it
			    #agiain with the same name just copy that file. Later on
			    #the lcc stats log file will be added to this file.
				$copyCommand = "copy ".$item." .";
				system $copyCommand; 
			}
			else
			{
				$fileList = $fileList." ".$item;
				&LogMessage($inv_file, "\t" . $item);
			}

      }
      $zipCommand = "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\zip.exe $zipFileName -m -j -g -q  $fileList";
      system $zipCommand;
   }
 }
 #iterate over the files to be copied and copy them into the specified zip file
 foreach $zipName (keys %zipToCopyFilesHash) 
 {
    if(not $zipName =~ /top/)
    {
      $zipFileName = $operatingHash{zip_identifier}.$zipName;
	  #Add key to the inventory file
	  &LogMessage($inv_file, "\t" . $zipFileName);
      $fileList = "";
      foreach $item (@{$zipToCopyFilesHash{$zipName}})
      {
        $fileList = $fileList." ".$item;
        &LogMessage($inv_file, "\t" . $item);

      }
      $zipCommand = "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\zip.exe $zipFileName -j -g -q  $fileList";
      system $zipCommand;
   }
 }
}

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CreateCollection
# Description: This function processes the files in the working directory and creates final zip
#------------------------------------------------------------------------------------------

sub CreateCollection()
{
    my (@dirContent, $finalZipList, $item, $zipCommand, $result, $zipName, $inv_file, $finalCopyList);
    chdir $dumpdir;
    opendir(DIR, $operatingHash{working_dir});
    @dirContent = grep(/(\w\S+)/g,readdir(DIR));
    closedir(DIR);
    $finalZipList = "";
    foreach $item (@dirContent)
    {
      #by pass the sub dir entries
      if(not grep($_ eq $item, @data_gather_sub_dirs))
      {
        $finalZipList = $finalZipList." ".$operatingHash{working_dir}."\\".$item;
      }
    }
    $inv_file = $operatingHash{working_dir}."\\".&GenUniqueLogName("zip_inventory.txt");
    #now get all the files to be copied into the top level zip
    $finalCopyList = "";
    foreach $zipName (keys %zipToCopyFilesHash) 
    {
      if($zipName =~ /top/)
      {
        foreach $item (@{$zipToCopyFilesHash{$zipName}})
        {
          $finalCopyList = $finalCopyList." ".$item;
          &LogMessage($inv_file, "\t" . $item);

        }
     }
    }    
    $zipCommand = "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\zip.exe $collectName -j -g -q  $finalCopyList";
    $result = `$zipCommand`;

    
    #now get all the contents of the top level zip created files
    foreach $zipName (keys %zipToFilesHash) 
    {
      if($zipName =~ /top/)
      {
        foreach $item (@{$zipToFilesHash{$zipName}})
        {
          $finalZipList = $finalZipList." ".$item;
          &LogMessage($inv_file, "\t" . $item);

        }
     }
    }    
    $zipCommand = "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\zip.exe $collectName -m -j -g -q  $finalZipList";
    $result = `$zipCommand`;
    
}

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CleanUp
# Description: This function removes the working directory.
#------------------------------------------------------------------------------------------
sub CleanUp()
{
   my ($subDir);
   unlink $dumpdir."\\".$operatingHash{log_file};
   foreach $subDir (@data_gather_sub_dirs)
   {
     unlink glob($operatingHash{working_dir}."\\".$subDir."\\*");
     rmdir $operatingHash{working_dir}."\\".$subDir;
   }
   chdir $dumpdir;
   rmdir $operatingHash{working_dir};

}


###########################################################################################
##########                               main                                    ##########
###########################################################################################


#Process command line
&ProcessCommandLine();

if($argHash{'cleanup'})
{
  &CleanUpOldRuns();
}
else
{

  #Initialize internal data structures
  &InitValues();

  #Set up the SPC name and working dir
  &SetZipIdentifierWorkingDir("spcollect_" . $operatingHash{file_date});

  #Create top level log containing version and timestamp info
  &GatherVersionInfo();

  #Output the name of the Collect file
  print "\n\nThe SP collect filename will be\n";
  print $operatingHash{zip_identifier} . "data.zip"."\n";

  #set up the zip hashes
  &SetUpZipHashes();

  #read the ini file and add the files to the appropriate zip file
  &ProcessIniFile();

  &LogMessage($operatingHash{fname_name_rev}, "End time of operation:" . &FilenameDT());
  
  #use the zip hashes and create the final zip file
  &CreateSubZips();

  #Gather SPCollect application log files
  &CollectPeerwatchAndSPCollectLog();

  &RunLogIt("To start zip ktlog files");
  &ZipKtLog();

  #Create the final zip
  &CreateCollection();

  #Clean up directory
  &CleanUp();
  
  &ListFiles();
}

###########################################################################################
##########                               end main                                    ##########
###########################################################################################
